import time, shutil
from bglobals import *
if not public:
    from other.learn_language import match_words, interlinear_translation
    from other.filter_txt import elim_end_hyphen

'''
"Sḗmĭs."1 ăn,1.1 hǣ́c3.5 ănĭmṓs9 ǣrū́go;4 ‿ēt5 cū́ră6 pĕcū́li;7
Cū́m2 sĕmĕl3 ī́mbŭĕrī́t,;8 spērḗmūs9 cā́rmĭnă12 fī́ngi11
Pṓssĕ10 lĭnḗndă;12 cĕdro;13 ‿ḗt14 lēuī́;16 sēruā́ndă15 cŭprḗsso?17 

in this passage we need an error message for two words with same number

'''


def help():
    p(f"""
    
    the file must be in downloads and there must be an accompanying
        file ending in _eng
    
    nw - to add new words to the vocab doc located in /latin/vocab
        doc will gather words once __start is found and stop once zzz
        is found.  automatically removes all numbers from words
        words must have ; in them to count
        to add a whole line then put all the words on that line and end
            it with ;;
        
    nw2 - adds news words to vocab but here the words must have @ in them
    
    awi - adds new words but the text is in interlinear format, words
        must have @ in them
        
    
    
    """)


def running_check(file):
    while 1:
        ins = match_words()
        ins.file = file
        ins.loop_til_right(1)
        ins = interlinear_translation()
        ins.file = file
        ins.loop_til_good()
        p('checked')
        time.sleep(60)


class phi:
    def __init__(self):
        pass

    def begin(self):
        self.get_atts()
        self.divide_seneca()
        self.shorten_auth_names()
        self.step1()
        self.move_phi()
        self.combine_works()
        # self.get_auth2words()
        # self.get_auth2words2()
        self.word2age()



    def get_atts(self):
        self.auth2work = defaultdict(dict)
        self.auth2num = defaultdict(dict)
        self.auth2words = defaultdict(dict)
        self.auth2date = {}
        self.auth2abb = {}
        self.poetic_lines = []
        self.for_move = defaultdict(list)
        self.unknown_dates = {}
        self.poets = set()
        self.phi_freq = defaultdict(int)
        self.poet2work = defaultdict(dict)
        self.poet2work2 = defaultdict(dict)

    def step1(self):
        poems_w_hyphen = to.from_txt2lst(f'{fold}phi_poems_hyphen')
        b = 0
        for auth in os.listdir(f'{phi_fold}'):
            oauth = auth
            if oauth == 'Publius Ovidius Naso':
                bb=8
            auth = self.auth2abb.get(auth,auth)
            auth3 = auth
            authp = f'{auth}_poet'


            if auth[0] != '.':
                file = f'{phi_fold}{oauth}'
                for work in os.listdir(file):
                    if not work == ds:
                        auth = auth3
                        b += 1
                        work = work[:-4]

                        poem = 0
                        vgf.print_intervals(b, 5, None, 800)
                        file2 = f'{file}/{work}'
                        lst = to.from_txt2lst(file2)
                        self.for_move[oauth].append((auth, work))
                        only_poetry = 0
                        self.auth2work[auth][work] = lst

                        if not auth in self.poets:
                            only_poetry = self.is_poetry(lst, work)

                        auth = self.change_auth.get((work,auth), auth)
                        if work[:-4].lower() in poems_w_hyphen:
                            lst = self.remove_hyphen(lst)
                        if auth in self.poets:
                            perc = self.is_poetry(lst,work,1)
                            if perc > 5:
                                bb=8
                            poetry = lst
                            prose = []
                            poem=1
                        elif '_poet' in auth:
                            poetry = lst
                            prose = []
                            poem=1

                        elif only_poetry:
                            poetry = lst
                            prose = []
                            poem = 1
                        else:
                            poetry, prose = self.divide_poet_prose(lst)
                            prose = self.remove_hyphen(prose)

                        for e, lst in en([prose, poetry]):
                            if lst:
                                if auth in self.poets:
                                    auth2 = auth
                                elif e:
                                    auth2 = authp
                                    poem = 1
                                elif poem:
                                    auth2 = authp
                                elif '_poet' in auth:
                                    auth2 = authp
                                    poem = 1
                                else:
                                    auth2 = auth
                                if poem:
                                    self.poet2work2[auth2][work] = lst

                                dct, total = self.small_normalize(lst, poem)
                                if not total:
                                    bb=8
                                self.auth2num[auth2][work] = total
                                self.auth2words[auth2][work] = dct
                                if poem:
                                    self.poet2work[auth2][work] = total

        self.auth2work = sort_dct_key(self.auth2work)
        self.auth2num = sort_dct_key(self.auth2num)
        self.poet2work = sort_dct_key(self.poet2work)
        return

    def combine_works(self):
        self.auth2words_com = {}
        d = 0
        for auth, works in self.auth2words.items():
            dct = defaultdict(int)
            for work, dct1 in works.items():
                for word, num in dct1.items():
                    dct[word] += num
                    self.phi_freq[word] += num
                    d += num
            self.auth2words_com[auth] = dct
        self.total = sum(self.phi_freq.values())
        tot = vgf.good_numbers(self.total,2)
        pl = set(self.poetic_lines)
        p (f'total words {tot}')
        p (f'total poetic lines {len(self.poetic_lines)}')
        p (f'without repeats {len(pl)}')
        return


    def poetry_database(self): #175
        pass

    def is_poetry(self, lst,name, ratio=0):
        '''
        this algorithm doesnt completely work for those authors
        which are extremely fragmentary since most fragments do
        not have hyphens, but right now we're only worried about
        authors with more than 5000 words
        '''

        titles = ['carmen','tragoediae','palliatae', 'praetexta',
                  'praetextae','carmina','tragoedia','togata','comoed']

        tot = len(lst)
        b=0
        for x in lst:
            if x.endswith('-'):
                b += 1
        perc = vgf.percent(b,tot)
        if ratio:
            return perc


        if perc < 2 and b < 3:
            if len(lst)< 50:
                for tit in titles:
                    if tit in name.lower():
                        return 1
                return 0
            else:
                return 1
        else:
            return 0


    def divide_poet_prose(self, lst):
        poem_indent = chr(160) + chr(160) + chr(160) + chr(160)
        poem_indent2 = chr(160) + chr(160) + chr(160)
        prose = []
        poetry = []
        for x in lst:
            if x.startswith(poem_indent2):
                poetry.append(x)
            else:
                prose.append(x)
        return poetry, prose

    def divide_seneca(self):
        file = f'{fold}seneca_divide'
        lst = to.from_txt2lst_tab_delim(file)
        self.change_auth = {}
        for l in lst:
            work = l[0]
            bool1 = 0
            try:
                bool1 = l[1]
            except:
                pass
            if bool1 == 1:
                self.change_auth[(work[:-4], 'seneca')] = 'seneca_poet'
            elif bool1 == 2:
                self.change_auth[(work[:-4],'seneca')] = 'seneca_pseudo'


        self.poets.add("seneca_pseudo")
        self.poets.add("seneca_poet")


    def small_normalize(self, lst, poem=0):
        words = []
        dct = defaultdict(int)
        for x in lst:
            if x and x[0] == '#':
                pass
            else:
                ox = x
                x = re.sub(r'[^a-zA-ZáéúÁÉÓÍÚíóāēīōūȳăĕĭŏŭўĀĒĪŌŪȲ\s]', '', x)
                x = unidecode(x)
                x = jv.replace(x)
                x = x.lower()
                x = x.strip()
                if x == 'char gl k':
                    bb=8
                if poem:
                    self.poetic_lines.append(x)
                words += x.split()
        for x in words:
            dct[x] += 1
        total = sum(dct.values())
        return dct, total

    def remove_hyphen(self, lst):
        lst = [x for x in lst if not x[0] == '#']
        for e, x in en(lst):
            lst[e] = x.replace(' *', '*')

        for b in range(len(lst) - 1):
            x = lst[b]
            y = lst[b + 1]
            if x and x[-1] == '-':
                y = y.split()
                yu = y[0]
                y = ' '.join(y[1:])
                x = x[:-1] + yu
                lst[b] = x
                lst[b + 1] = y

        return lst

    def get_auth2words(self):
        self.long_works = {}
        for auth, works in self.auth2num.items():
            for work, num in works.items():
                self.long_works[(auth, work)] = num

        self.long_works = sort_dct_val_rev(self.long_works)
        self.auth2work = sort_dct_key(self.auth2work)
        b = 0
        tot_words = 0
        for x, y in self.long_works.items():
            tot_words += y
            if y > 5000:
                b += 1

    def get_auth2words2(self):
        dct = defaultdict(int)
        for k, v in self.auth2num.items():
            num = 0
            for x, y in v.items():
                num += y
            dct[k] = num
        dct = sort_dct_val_rev(dct)
        self.auth2num2 = dct
        # 99.69, 7.73 million words
        b = 0
        total = 0
        d = 0
        self.focus = set()
        for k, v in dct.items():
            if v > 500:
                self.focus.add(k)
                b += 1
                d += v
            total += v
        p(d, total, d / total)

    def shorten_auth_names(self):
        # u2l = {k.lower(): k for k in self.auth2num.keys()}
        u2l = {k.lower(): k for k in os.listdir(f'{phi_fold}') if k != ds}
        lst1 = to.from_txt2lst_tab_delim(f'{fold}phi_auth2')
        lst = to.from_txt2lst_tab_delim(f'{fold}phi_auth', 0, 1)
        lst2 = []
        for g, x in en(lst1):
            if x[0] == '__stop':
                break
            lst2.append(x)
        lst2 += lst
        # because of the function divide_seneca he is given a special case
        lst2 = [x for x in lst2 if x[0] != 'Lucius Annaeus Seneca iunior']
        miss = set()
        start = g
        done = set()
        for g, l in en(lst2):
            oname = l[0]
            if type(oname) != int:
                if oname.lower() == 'iustinianus':
                    bb = 8
                udate = None
                if oname == 'Publius Ovidius* Naso':
                    bb = 8
                name = oname
                name = name.replace('*', '')
                if name.lower() not in done:
                    done.add(name)
                    if g < start:
                        fname = u2l.get(name)
                        if fname == None:
                            bb = 8
                    else:
                        fname = name
                    birth = None
                    death = None
                    other = None
                    other2 = None
                    try:
                        birth = l[1]
                        death = l[2]
                        other = l[3]
                        other2 = l[4]
                    except:
                        pass
                    if birth and type(birth) == str and "?" in birth:
                        udate = birth.replace('?', "")
                        udate = int(udate)
                        birth = None
                    if birth == '':
                        birth = None
                    if death == '':
                        death = None
                    if other == '':
                        other = None
                    if other2 == '':
                        other2 = None

                    oname2 = oname
                    if type(death) == str:
                        oname2 += f' {death}'
                    if other and other != 'po':
                        oname2 += f' {other}'
                    if '*' in oname2:
                        aname = oname2.split()
                        aname = [x[:-1] for x in aname if '*' in x]
                        aname = ' '.join(aname)
                    else:
                        aname = name
                    aname = aname.lower()
                    if udate != None:
                        self.unknown_dates[aname] = udate
                    if fname not in self.auth2abb:
                        self.auth2abb[fname] = aname
                    if other == 'po' or other2 == 'po':
                        self.poets.add(aname)
                    if death == 'po':
                        self.poets.add(aname)
                        death = None


                    approx = None
                    approx2 = None
                    if birth != None:
                        if type(death) == str:
                            death = None

                        if birth == '_':
                            birth = None
                        if birth != None and death == None:
                            death = birth
                            birth = None

                        if birth != None and death != None:
                            approx = birth + 30
                        elif death != None:
                            if death % 50 == 0:
                                approx = death
                            else:
                                approx = death - 20

                        if approx != None:
                            if approx % 50 == 0:
                                approx2 = approx
                            else:
                                b = approx // 50
                                rem = approx % 50
                                if approx > 0 and rem > 25:
                                    b += 1
                                if approx < 0 and rem > 25:
                                    b += 1

                                approx2 = b * 50
                    if approx2 != None:
                        if aname not in self.auth2date:
                            self.auth2date[aname] = approx2
        assert 'ovidius' in self.auth2date
        assert 'iustinianus' in self.auth2date
        return


    def move_phi(self):
        b = 0
        for k,v in self.for_move.items():
            for z in v:
                b += 1
                vgf.print_intervals(b,5,None,800)
                auth, work = z
                work2 = work.lower()
                fold2 = f'{phi_fold2}{auth}'
                if not os.path.exists(fold2):
                    os.mkdir(fold2)
                src = f'{phi_fold}{k}/{work}.txt'
                dest = f'{phi_fold2}{auth}/{work2}.txt'
                shutil.copy(src, dest)




    def word2age(self):
        self.auth2num2ab = {self.auth2abb[k]: v for k, v in self.auth2num2.items()}
        self.auth2date['cicero']=-75
        self.auth2date['cicero_poet']=-75
        self.age2word = defaultdict(int)
        for x, y in self.auth2date.items():
            obj = self.auth2num2ab.get(x)
            if obj:
                self.age2word[y] += obj
        for x, y in self.age2word.items():
            self.age2word[x] = vgf.good_numbers(y, 2)
        self.age2word = sort_dct_key(self.age2word)
        self.auth2date = sort_dct_val(self.auth2date)
        self.poets = list(self.poets)
        self.poets.sort()

        return


def extract_rearrange(file):
    '''
This will take every line in an _itrans document and put it on a new file so that you can
read the rearrangement with ease.  in the _itrans doc every latin line that ends with ,, is
followed by a rearranged line

    '''

    file1 = f'{dwn_dir}{file}'
    file2 = f'{dwn_dir}{file}'
    t = 'ī́'
    p(ord(t[-1]))

    file2 = file2.replace('_itrans', '') + '_rearr'
    lst = to.from_txt2lst(file1, 1)
    lst1 = []
    for e, x in en(lst):
        if x and x[0].isdigit():
            num = x[:x.index(' ')]

        if x.endswith(',,'):
            s = lst[e + 2]
            s = unidecode(s)
            s = re.sub(r'[\",\.\?]', '', s)
            s = s.lower()
            lst1.append(f"{num} {s}")
            lst1.append('')
    to.from_lst2txt(lst1, file2)
    vgf.open_txt_file(file2)


def focus(file, only_list=0):
    '''
this function takes a text in the itrans format in which
there is a number beginning the line of each foreign language
line, followed by rearrangement, then english and so long
as there is a / in the line then it will save that line alone
with the rearrangement and the english and put it on another
file.

    '''

    file1 = f'{dwn_dir}{file}'
    file3 = f'{dwn_dir}{file}a'
    file4 = f'{dwn_dir}{file}b'
    lst = to.from_txt2lst(file1, 1)
    lst1 = vgf.clump_lst(lst, 1)
    lst3 = []
    lst4 = []
    nums = []
    on = 0
    for x in lst1:
        y = x[0]
        num = y[:y.index(' ')]
        if any('/' in z for z in x):
            nums.append(num)
            lst4.append(x)
            on = 1
        elif x[0][0].isdigit():
            if lst4:
                lst3.append(lst4)
            lst4 = []
            on = 0
        elif on:
            lst4.append(x)

    if lst4:
        lst3.append(lst4)

    if only_list:
        return lst3

    lst5 = []
    lst6 = []
    for x in lst3:
        for z in x:
            for y in z:
                lst5.append(y)
                lst6.append(y)
            lst5.append('')

    file2 = f'{dwn_dir}nums'
    to.from_lst2txt(nums, file2)
    to.from_lst2txt(lst5, file3)
    to.from_lst2txt(lst6, file4)
    vgf.open_txt_file(file2)
    return


class lasla_pos_txt(phi):
    def __init__(self):
        phi.__init__(self)

    def begin(self):  # 200013
        self.get_atts()
        # self.simplify_old()
        self.any_author()
        self.stats()
        # self.add_elision()
        # self.temp_text()
        self.output()

    def get_atts(self):
        self.lasla_db2 = pi.open_pickle(f'{fold}lasla_db2')
        self.las_freq = pi.open_pickle(f'{fold}las_lem_freq')
        self.co_lemmas4 = pi.open_pickle(f'{fold}co_lemmas4')
        self.llemclem = pi.open_pickle(f'{fold}llem2clem2')
        self.noun2gender = pi.open_pickle(f'{fold}lasla_noun2gen')
        self.lem2forms_rev_ui = pi.open_pickle(f'{fold}lem2forms_rev')
        self.lem2forms_rev_jv = pi.open_pickle(f'{fold}lem2forms_rev_jv')
        self.old_wonum = pi.open_pickle(f'{fold}old_wdef')
        self.old_simple = pi.open_pickle(f'{fold}old_simple')
        self.old_variant = pi.open_pickle(f'{fold}old_var2parent')
        self.miss_def = {}
        self.kind = 'pet'
        self.auth = 'Petronius'
        self.tpos = {
            'C': 'm',
            'X': 'm',
            'B': 'm',
        }

    def research_ui(self):
        self.old_wonum = pi.open_pickle(f'{fold}old_wdef')
        dct = {}
        for k, v in self.old_wonum.items():
            if 'ui' in k and not 'qui' in k:
                dct[k] = v.wmacron
        dct1 = {}
        for k, v in self.old_wonum.items():
            if 'ui' in k and not 'qui' in k:

                if hasattr(v, 'wmacron') and tie in v.wmacron:
                    dct1[k] = v.wmacron

    def simplify_old(self):
        dct = defaultdict(dict)
        for k, z in self.old_wonum.items():
            ku, num = cut_num(k, 1)
            pos = " ".join(z.pos)
            s = f'{z.wmacron} {pos} {z.inflection}'
            t = ''
            for x, y in z.defs.items():
                if y:
                    t += f"{x} {y}"
            dct[ku.lower()][num] = [s, t]

        b = 0
        for k, v in dct.items():
            if len(v) > 1:
                b += 1
        p(b)
        pi.save_pickle(dct, f'{fold}old_simple')
        self.old_simple = dct
        return

    def any_author(self):
        self.lst = []
        self.lidx = ''
        self.bad_pos = set()
        self.tgen = 0
        self.missing = defaultdict(int)
        self.hgen = 0
        self.tmac = 0
        self.gdef = 0
        self.error = []
        self.hmac = 0
        for k, books in self.lasla_db2[self.auth].items():
            self.aut_book = k
            if k == 'PetroniusSatiricon_PetronSa':
                self.books = books
                for e, v in en(books):
                    vgf.print_intervals(e, 500, None, len(books))
                    self.idx = v[3]
                    self.lem = v[1]
                    self.lemu, self.num = cut_num(self.lem, 1)
                    self.lemu = self.lemu.lower()
                    self.cat = v[0]
                    freq = self.las_freq.get(self.lem.lower(), 0)
                    self.get_defn(freq)
                    self.word = v[2]
                    self.word_jv = ''
                    self.gender = self.noun2gender.get(self.lem, '')
                    self.pos = v[4]
                    self.pos2 = ''
                    self.noun_gender()
                    if self.cat != '!' and self.pos not in ['_gr']:
                        self.tmac += 1
                        if self.kind != 'simple':
                            self.add_macrons()
                    if self.lidx != self.idx:
                        self.lst.append([self.idx])
                    if not self.word_jv:
                        self.missing[(self.lem, self.word, self.pos)] += 1
                    if self.pos2:
                        self.pos = self.pos2
                    if not self.word_jv:
                        self.word_jv = self.word + circle
                    v.append(self.word_jv)
                    if self.defn:
                        itm = vgf.dct_idx(self.defn, 0, 'v')
                        self.lst.append(['', self.lem, self.word,
                                         self.word_jv, freq, self.pos,
                                         itm[0], itm[1]])

                        if len(self.defn) > 1:
                            h = 0
                            for g, r in self.defn.items():
                                if h > 0:
                                    self.lst.append(['', '', '', '', '', '', r[0], r[1]])

                                h += 1

                    else:
                        self.lst.append(['', '', self.lem, self.word, self.word_jv, self.pos])
                    self.lidx = self.idx
                if self.kind == 'pet':
                    self.output(1)
        self.missing = sort_dct_val_rev(self.missing)
        self.miss_def = sort_dct_val_rev(self.miss_def)
        return

    def get_defn(self, freq):
        obj = self.old_simple.get(self.lemu)
        if not obj:
            if self.lemu in encs:
                self.gdef += 1
            else:
                obj = self.old_variant.get(self.lemu)
                if not obj and self.num not in ['8', '9']:
                    self.miss_def[self.lemu] = freq
                elif obj:
                    self.gdef += 1
                    lst = []
                    for k, v in obj.items():
                        vu, num = cut_num(v, 1)
                        try:
                            lst.append(self.old_wonum[vu][num])
                        except:
                            self.error.append(self.lemu)
                            pass
                    if lst:
                        self.defn = lst



        else:
            self.defn = obj
            self.gdef += 1

    def temp(self):
        dct8 = {}
        for x, y in dct7.items():
            st = set()
            for k, v in y.items():
                vu = cut_num(v)
                st.add(vu)
            if len(st) > 1:
                dct8[x] = y

    def temp_text(self):
        lin = ''
        lst1 = []
        b = 0
        for e, x in en(self.lst):
            vgf.print_intervals(e, 100, None, len(self.lst))
            if len(x) == 1:
                if e and lin:
                    lst1.append(lin)
                    lin = ''
                num = x[0].strip()
                if num.endswith(',1'):
                    lst1.append('')
                    lst1.append(f'CAPUT {num[:-2]}')
                    lst1.append('')

            else:
                word = x[2]
                if word[0] == el:
                    lin += word
                else:
                    lin += " " + word
            if len(lin) > 50:
                pts, _ = vgf.get_text_size(lin)
                if pts > 467:
                    lst1.append(lin)
                    lin = ''
        file = f'{dwn_dir}petronius_mac'
        to.from_lst2txt(lst1, file)
        vgf.open_txt_file(file)
        return

    def enclitic_stress(self):
        """
        When an enclitic is joined to a word, the accent falls | on the syllable next before the enclitic, whether long or short: as, dĕă'que,\
which words retain the accent of the complete words which have lost a vowel | Certain words which have lost a final vowel retain the accent of the complete words: as, illī'c forillī'ce, prōdū'c for prōdūce, sati'n for sati'sne.\

name two exceptions to accent rules | 1 Certain apparent compounds of faciō retain the accent of the simple verb. 2. In the second declension the genitive and vocative of nouns in -ius and the genitive of those in -ium retain the accent of the nominative\

If the vowel or syllable ending in -m is long by nature, it usually will not be elided (so for instance in Demosthenem the final syllable will always be pronounced, even if it comes before a vowel.

For the most part, a long vowel at the end of a word coming before a vowel at the beginning of the next word is elided; but in the case of o, u and i, it sometimes happens that the result is like the addition of a consonant, v or j:  immo age for instance sounds like “im-mwa-ge.”  See Allen’s Vox Latina for more details.

        """
        pass

    def line1(self):
        lst = ['lem', 'word', 'pos']

        dct = {x: vgf.get_text_size(getattr(self, x)) for x in lst}

    def add_macrons(self):
        if len(self.pos) == 2 and self.pos[0] in ['s', 'p']:
            self.tgen += 1
            if self.gender:
                self.hgen += 1

        special = 0
        pos2 = self.pos
        if reg(r'[CBX]', self.pos):
            for x, y in self.tpos.items():
                pos2 = pos2.replace(x, y)
        elif reg(r'[A-Z]', self.pos):
            special = 1

        lem_wn, num = cut_num(self.lem, 1)
        obj = self.llemclem.get(lem_wn)

        if obj:
            cnum = obj.get(num)
            if cnum != None:
                itm = self.lem2forms_rev_jv.get(lem_wn + cnum)
                itm2 = self.lem2forms_rev_ui.get(lem_wn + cnum)
                mword = 0
                mword_ui = 0
                if itm:
                    if special:
                        self.infer_pos(itm2, itm)
                    else:

                        words = itm.get(pos2)
                        words_ui = itm2.get(pos2)
                        if words:
                            mword = words
                            mword_ui = words_ui

                        elif vgf.dct_idx(itm) == 'inv':
                            mword_ui = itm2['inv']
                            mword = itm['inv']
                        else:
                            self.bad_pos.add(self.pos)

                if mword_ui:
                    try:
                        idx = mword_ui.index(self.word)
                        self.word_jv = mword[idx]
                        self.hmac += 1
                    except ValueError:
                        pass

        return

    def infer_pos(self, itm, itm_jv):
        '''
        for now we are just going to use one word
        not all of them, same with pos
        '''
        done = 0
        for k, v in itm.items():
            try:
                idx = v.index(self.word)
                if not done:
                    self.word_jv = itm_jv[k][idx]
                    done = 1
                    self.hmac += 1
                self.pos += k + ' '
            except ValueError:
                pass

    def noun_gender(self):
        if self.gender:
            self.pos2 = add_at_i(1, self.pos, self.gender)

    def stats(self):
        num = round(self.hmac / self.tmac, 2)
        gnum = round(self.hgen / self.tgen, 2)
        dnum = round(self.gdef / self.tmac, 2)

        p(f'macrons: {num} gender: {gnum} definition: {dnum}')

    def output(self, kind=0):
        if kind == 1:
            file = f'{lfold}lasla2/{self.aut_book}'
            to.from_lst2txt_tab_delim(self.books, file)

        # file = f'{dwn_dir}petr'
        # for e, x in en(self.lst):
        #     try:
        #         self.lst[e][2] = x[2].replace(el, '')
        #     except:
        #         pass
        #
        # ef.from_lst2book(self.lst, file)
        # ef.open_wb(file)
        return


'''
things we have don't yet for the below class
mid punctuation is still not coded for, so
[ ] < > can appear within a word
for those words which are not matched to phi
if they have a j or v it will be put back on
better to make two texts, one without [ ] < >
C. is getting rendered as C..
should probably replace numbers with words such as III

some quotes can be put on the wrong words


'''


class punctuate_lasla(lasla_pos_txt):
    '''
    the enclitics are not getting printed

    '''

    def __init__(self):
        lasla_pos_txt.__init__(self)

    def begin(self):
        self.get_atts()
        self.normalize_las()
        self.elim_enc()
        self.phi_lst = self.remove_hyphen(self.phi_lst)
        self.normalize_phi()
        self.normalize_phi2()
        self.match_phi2las()
        self.add_punc()
        self.add_elision()
        self.add_paragraphs()

    def get_atts(self):
        self.kind = ''
        if public:
            self.phi_lst = to.from_txt2lst(f'{fold}satyrica')
        else:
            self.phi_lst = to.from_txt2lst(f'{phi_fold2}petronius/satyrica')
        # if self.kind != 'all':
        #     self.lasla_db2 = pi.open_pickle(f'{fold}lasla_db2')
        # self.ltxt = self.lasla_db2['Petronius']['PetroniusSatiricon_PetronSa']
        file = f'{lafold2}PetroniusSatiricon_PetronSa'
        self.ltxt = to.from_txt2lst_tab_delim(file, 1, 0)
        self.normalize_las()
        self.ltxtw = [x[2].strip() for x in self.ltxt]
        self.ltxtw = [x.lower() for x in self.ltxtw]

    def normalize_las(self):
        '''
        get rid of the |st in the j_lasla2 file
        '''

        self.ltxt = [x for x in self.ltxt if x[4] not in ['_rf', "_rp", '_nsp']]
        for e, x in en(self.ltxt):
            if '|st' in x[2]:
                self.ltxt[e][2] = x[2][:-3]
                self.ltxt[e + 1][2] = 'est'
            if '|s' in x[3]:
                self.ltxt[e][2] = x[2][:-1]
                self.ltxt[e + 1][2] = 'es'
        return

    def normalize_phi(self):
        poem_indent = chr(160) + chr(160) + chr(160) + chr(160)
        poem_indent2 = chr(160) + chr(160) + chr(160)
        para_indent = chr(160) + chr(160)
        one = '¹'
        four = '⁴'

        for e, x in en(self.phi_lst):
            x = x.replace(' . . .', '$ ')

            if 'agri iacent' in x:
                bb = 8
            if poem_indent2 in x:
                x += '+'
            if poem_indent in x:
                x = re.sub(r'' + chr(160) + '{4,}', "@", x)

            x = x.replace(poem_indent, '@')
            x = x.replace(poem_indent2, '&')
            x = x.replace(para_indent, '%')
            x = x.replace(" *", '*')
            x = x.replace('—', ', ')
            x = x.replace(', ,', ', ')
            x = x.replace(';', ',')
            x = re.sub(r'[' + one + four + ']', '', x)
            x = re.sub(r'\s{2,}', ' ', x)
            x = x.rstrip()
            self.phi_lst[e] = x.lower()

        return

    def normalize_phi2(self):
        '''
        for bpunc which can be either on the beginning
        of the word or the end, we will have to make it
        such that every odd indexed bpunc character is on the
        left and every even indexed bpunc is on the right

        '''

        punctuation = ',.{}"():!?[]<>†$*@%&+' + "'"
        self.punctuation = punctuation
        lpunc = '{([<@%&'
        rpunc = '})]>*+,.:!?"$'
        assert not set(lpunc) & set(rpunc)
        bpunc = '†"' + "'"
        self.phi_lst2 = []
        for x in self.phi_lst:
            self.phi_lst2 += x.split()

        for e, x in en(self.phi_lst2):
            xo = re.sub(r'[,\.\{\}"\(\):\+!\?\[\]\<\>†\$@&%\'\*]', '', x)
            if not xo:
                if x in lpunc:
                    self.phi_lst2[e + 1] = x + self.phi_lst2[e + 1]
                    self.phi_lst2[e] = ''
                elif x in rpunc or x in bpunc:
                    self.phi_lst2[e - 1] = x + self.phi_lst2[e - 1]
                    self.phi_lst2[e - 1] = ''
        self.phi_lst2 = [x for x in self.phi_lst2 if x]
        self.lpunc_dct = {}
        self.rpunc_dct = {}
        for e, x in en(self.phi_lst2):
            s = ""
            xo = x
            for z in x:
                if z in punctuation:
                    s += z
                else:
                    break
            if s:
                self.lpunc_dct[e] = s

            s = ""
            for z in reversed(x):
                if z == "*":
                    bb = 8

                if z in punctuation:
                    s = f'{z}{s}'
                else:
                    break
            if s:
                self.rpunc_dct[e] = s
                if s == '@' and self.lpunc_dct[e] == '@':
                    bb = 8

            x = re.sub(r'[,\.\{\}"\(\):\+!\?\[\]\<\>†\$@&%\'\*]', '', x)
            x = jv.replace(x)
            if not x:
                bb = 8
            self.phi_lst2[e] = x.lower()

    def elim_enc(self):
        b = len(self.ltxtw) - 1
        b = 0
        olst = copy.deepcopy(self.ltxtw)
        self.idx2enc = {}
        self.idx2enc2 = {}
        while b < len(self.ltxt):
            word = self.ltxt[b][2]
            lem = self.ltxt[b][1]
            if word == 'titulos':
                bb = 8
            lst = self.ltxt[b]
            if word in ['que', 'ue'] or lem == 'ne2':
                self.ltxtw[b - 1] += word
                self.idx2enc[b - 1] = word
                self.idx2enc2[b - 1] = lst
                del self.ltxtw[b]
                del self.ltxt[b]
            else:
                b += 1
            bb = 8
        # c = 0
        # for idx, word in self.idx2enc.items():
        #     word1 = self.ltxtw[idx + c]
        #     self.ltxtw[idx + c] = word1[:-len(word)]
        #     self.ltxtw.insert(idx + 1 + c, word)
        #     bb = 8
        #     c += 1

        return

    def spaced(self, pidx):
        b = self.lword.count(' ')
        if b == 2:
            pnxt3 = " ".join(self.phi_lst2[pidx:pidx + 3])
            if self.lword == pnxt3:
                return pidx + 2
        elif b == 1:
            pnxt2 = " ".join(self.phi_lst2[pidx:pidx + 2])
            if self.lword == pnxt2:
                return pidx + 1

    def isgreek(self, x):
        xu = re.sub(r'[αβψδεφγηιξκλμνοπρστθωςχυζΑΒΨΔΕΦΓΗΙΞΚΛΜΝΟΠΡΣΤΘΩΥΖέἀῖëῖί]', '', x)
        if not xu:
            return 1
        if len(xu) / len(x) > .1:
            self.almost_greek.append(x)
            if not public:
                p(f'almost greek {x}')
            return 2
        self.should_be_greek.append(x)

    def match_phi2las(self):
        self.no_rpunc = []
        self.no_lpunc = []
        self.l2p = {}
        self.p2l = {}
        self.greek = {}
        self.mismatches = 0
        self.almost_greek = []
        self.should_be_greek = []
        pidx = 0
        lidx = 0
        while pidx < len(self.phi_lst2):
            if pidx == 251:
                bb = 8
            self.mirror = 0
            self.pword = self.phi_lst2[pidx].lower()
            self.lword = self.ltxtw[lidx].lower()
            if self.lword == 'etiam si':
                bb = 8
            if self.lword == 'quam ob rem':
                bb = 8
            pidx2 = self.spaced(pidx)
            if pidx2:
                pidx = pidx2
            else:
                if self.lword == 'greek' and self.isgreek(self.pword):
                    self.greek[lidx] = self.pword
                elif self.pword != self.lword:
                    dist = vgf.lvn.distance(self.pword, self.lword)
                    if dist > 2:
                        pidx, lidx = self.match_phi2las2(pidx, lidx)
            if not self.mirror:
                self.l2p[lidx] = pidx
                self.p2l[pidx] = lidx
            pidx += 1
            lidx += 1
        if not public:
            p(self.mismatches)
        return

    def match_phi2las2(self, pidx, lidx):
        '''
        mecum, tecum, secum, vobiscum, nobiscum
        - space in p not in l
        - mirror image
        mēcum
        '''

        pidx3, lidx3 = self.cum_enclit(pidx, lidx)
        if pidx3:
            return pidx3, lidx3
        pidx3, lidx3 = self.pspace(pidx, lidx)
        if pidx3:
            return pidx3, lidx3
        pidx3, lidx3 = self.mimage(pidx, lidx)
        if pidx3:
            return pidx3, lidx3

        nlst = []
        op = pidx
        ol = lidx
        for i in range(10):
            for j in range(10):
                if i == 0 and j == 0:
                    pass
                else:
                    nlst.append((i, j))
        found = 0
        for tpl in nlst:
            b, c = tpl
            pidx2 = pidx + b
            lidx2 = lidx + c
            pnxt3 = " ".join(self.phi_lst2[pidx2:pidx2 + 3]).lower()
            lnxt3 = " ".join(self.ltxtw[lidx2:lidx2 + 3]).lower()
            if pnxt3 == lnxt3:
                found = 1
                break

        if not found:
            p(self.phi_lst2[pidx:pidx + 20])
            p(self.ltxtw[lidx:lidx + 20])
            pidx2 = pidx + 7
            lidx2 = lidx + 7

        oddp = self.phi_lst2[op:pidx2]
        oddl = self.ltxtw[ol:lidx2]
        oddp = ' '.join(oddp)
        oddl = ' '.join(oddl)
        if oddl == 'etiam si':
            bb = 8
        if not oddp and not oddl:
            bb = 8

        if oddp == 'mecum':
            bb = 8

        if not public:
            p(f'p: {oddp} l: {oddl}')
            p('')
        self.mismatches += 1

        return pidx2, lidx2

    def cum_enclit(self, pidx, lidx):
        if pidx == 10718:
            bb = 8

        if self.pword in ['mecum', 'tecum', 'secum',
                          'uobiscum', 'nobiscum']:
            lnxt2 = " ".join(self.ltxtw[lidx:lidx + 2]).lower()
            lnxt2 = lnxt2.replace(' ', '')
            if lnxt2 == self.pword:
                self.no_rpunc.append(pidx)
                self.ltxt[lidx + 1][5] = f'^cum'
                return pidx, lidx + 1
        return 0, 0

    def pspace(self, pidx, lidx):
        if self.lword == 'inuicem':
            bb = 8

        pnxt2 = "".join(self.phi_lst2[pidx:pidx + 2]).lower()
        if pnxt2 == self.lword:
            self.no_rpunc.append(pidx)
            self.no_lpunc.append(pidx + 1)
            return pidx + 1, lidx
        return 0, 0

    def mimage(self, pidx, lidx):
        pnxt2 = " ".join(self.phi_lst2[pidx:pidx + 2]).lower()
        lnxt2 = f'{self.ltxtw[lidx + 1]} {self.ltxtw[lidx]}'
        if pnxt2 == lnxt2:
            self.p2l[pidx] = lidx + 1
            self.p2l[pidx + 1] = lidx - 1
            self.mirror = 1
            return pidx + 1, lidx + 1
        return 0, 0

    def add_punc(self):
        miss = []
        klst = list(self.p2l.keys())
        vlst = list(self.p2l.values())

        for k, v in self.lpunc_dct.items():
            num = self.p2l.get(k)
            if num == None:
                if v in '@&%':
                    bb = 8
                miss.append(k)
            else:
                word = self.ltxt[num][5]
                self.ltxt[num][5] = f'{v}{word}'

        for k, v in self.rpunc_dct.items():
            num = self.p2l.get(k)
            if num == None:
                if v in '+*$':
                    bb = 8
                miss.append(k)
            else:
                word = self.ltxt[num][5]
                self.ltxt[num][5] = f'{word}{v}'
        self.ltxt2 = [x[5] for x in self.ltxt]
        return

    def add_elision(self):
        c = 0
        for x, y in zip(self.ltxt2[:-1], self.ltxt2[1:]):
            try:
                xword = x
                yword = y

                if reg(r'[\?\!\.\*\+\$]', xword) or \
                        reg(r'[@%&]', yword):
                    pass
                elif self.quote(xword, yword):
                    pass
                else:
                    ywordu = unidecode(yword)
                    xwordu = unidecode(xword)
                    if reg(r'[āēīōū]m$', xword):
                        pass
                    else:
                        if ywordu not in ['et', 'o']:
                            if ywordu[0] in 'haeiouy':
                                if xwordu[-1] in 'aeiouy' or \
                                        reg(r'[aeioyu]m$', xwordu):
                                    self.ltxt2[c + 1] = el + yword



            except:
                pass
            c += 1

        return

    def quote(self, x, y):
        """
        if a word begins a quotation then it cannot
        be elided with a word outside the quotation
        """

        if "'" not in x and "'" not in y and \
                '"' not in x and '"' not in y:
            return 0
        for e, z in en(x):
            if z not in self.punctuation:
                xend = x[e:]
                break
        e = len(x) - 1
        for z in reversed(y):
            if z not in self.punctuation:
                ystart = y[:e]
                break
            e -= 1

        if "'" in xend or '"' in xend or \
                "'" in ystart or "'" in ystart:
            return 1

    def add_paragraphs(self):
        for e, x in en(self.ltxt2):
            self.ltxt2[e] = x.replace('$', ' ...')
        lst = []
        s = ''
        poem = 0
        for e, x in en(self.ltxt2):
            x = self.ltxt2[e]
            if x == 'plausor':
                bb = 8

            if "%" in x:
                lst.append(s.rstrip())
                x = x.replace('%', '  ')
                s = x + ' '
            elif "@" in x:
                if s:
                    lst.append(s.rstrip())
                if not poem:
                    lst.append('')
                x = x.replace('@', '    ')
                s = x + ' '
                poem = 1
            elif "&" in x:
                if s:
                    lst.append(s.rstrip())
                if not poem:
                    lst.append('')
                x = x.replace('@', '   ')
                s = x + " "
                poem = 1
            elif '+' in x:
                x = x.replace('+', '')
                s += x
                lst.append(s)
                s = ''
                try:
                    nxt = self.ltxt2[e + 1]
                except:
                    nxt = ''
                if not reg(r'[@&]', nxt):
                    lst.append('')
                    poem = 0
                else:
                    poem = 1
            else:
                s += x + ' '

        # we now erase the space between the
        # cum enclitic
        for e, x in en(lst):
            x = x.replace(' ^', '')
            x = x.replace('*', ' ....')
            x = x.replace(f' {el}', el)
            lst[e] = x

        file = f'{fold}satyrica_w_macrons'
        file2 = f'{phi_fold2}petronius/satyrica'
        to.from_lst2txt(lst, file)
        vgf.open_txt_file(file)
        vgf.open_txt_file(file2)
        return

        #
        #
        # if poem_indent2 in x:
        #     x += '+'
        # x = x.replace(poem_indent, '@')
        # x = x.replace(poem_indent2, '&')
        # x = x.replace(para_indent, '%')

    def observe(self):
        lk = -1
        lv = -1
        for k, v in self.l2p.items():
            if k != lk + 1:
                bb = 8
            if v != lv + 1:
                bb = 8

    def temp18(self):
        lst = []
        str1 = ''
        for x in self.ltxtw:
            str1 += x + ' '
            if len(str1) > 70:
                lst.append(str1[:-1])
                str1 = ' '
        file = f'{dwn_dir}pet_punc'
        to.from_lst2txt(lst, file)
        vgf.open_txt_file(file)
        return


class quick_lookup:
    def __init__(self):
        pass

    def begin(self, file):
        self.file = f'{dwn_dir}{file}'
        self.ofile = file
        self.get_atts()
        self.step1()

    def get_atts(self):
        self.def_lemmas = pi.open_pickle(f'{fold}def_lemmas', 1)
        self.macronizer = pi.open_pickle(f'{fold}macronizer_ncaps', 1)
        self.fake_enclitics = pi.open_pickle(f'{fold}fake_enclitics', 1)
        self.lem_freq_rank = pi.open_pickle(f'{fold}lem_freq_crude', 1)
        pass

    def step1(self):
        while 1:
            lst = to.from_txt2lst(self.file, 1)
            lst1 = []
            for x in lst:
                lst1 += x.split()
            for x in reversed(lst1):
                if ';' in x:
                    break
            self.look_up(x)

    def look_up(self, x):
        pass


class rename_phi:
    def __init__(self):
        pass

    def get_atts(self):
        self.phi_aut = os.listdir(f'{phi_fold}')
        file = f'{fold}phi_auth'
        to.from_lst2txt(self.phi_aut, file)
        vgf.open_txt_file(file)


class new_words:
    def __init__(self):
        pass

    def begin(self, file):
        self.file = file
        self.get_atts(file)
        self.step1()
        self.add2excel()
        self.export()

    def begin2(self, file):
        self.file = file
        self.get_atts(file, 'second')
        self.step1()
        self.add2excel()
        self.export(1)

    def begin_add(self, file):
        self.file = file
        self.get_atts(file, 'third')
        self.step1()
        self.add2excel()
        self.export(1)

    def get_atts(self, file, kind=0):
        self.def_lemmas = pi.open_pickle(f'{fold}co_lemmas5', 1)
        self.macronizer = pi.open_pickle(f'{fold}macronizer_ncaps', 1)
        self.fake_enclitics = pi.open_pickle(f'{fold}fake_enclitics', 1)
        self.lem_freq_rank = pi.open_pickle(f'{fold}lem_freq_crude', 1)
        p('done opening')
        ins = match_words()
        ins.remove_scansion = 1
        if kind == 'second':
            self.final_lst, self.exc_sh = ins.begin2(file, '@', 0, 'second')
        elif kind == 'third':
            self.final_lst, self.exc_sh = ins.begin2(file, ';', 0, 'third')
        else:
            self.final_lst, self.exc_sh = ins.begin2(file)
        return

    def step1(self, divided=1):
        for e, l in en(self.final_lst):
            if divided:
                lword = l[1]
            else:
                lword = l[0]
            if '_' not in lword:
                lwordu = norm_str_jv(lword)
                lwordu, enc = enclitics(lwordu, self.fake_enclitics)
                itm = self.macronizer.get(lwordu)
                if itm:
                    tot_def = []
                    for k, dct in itm.items():
                        for lemma, plst in dct.items():
                            pos = " ".join(plst)
                            lemu = norm_str_jv(lemma)
                            lemmacl = self.def_lemmas.get(lemu)
                            freq = self.lem_freq_rank.get(lemu, -1)
                            if not lemmacl:
                                p(f'missing lemmas {lemu} {l[0]}')
                            else:
                                defn = get_def(lemmacl, 1)
                                tot_def += [k, freq, lemma, pos, defn]
                    if tot_def:
                        l += tot_def
                    self.final_lst[e] = l
                else:
                    p(f'could not find {lwordu}')

        return

    def add2excel(self):
        self.exc_sh = self.exc_sh[1:]
        self.exc_sh = [x[1:] for x in self.exc_sh]

        for e, x in en(self.final_lst):
            y = [None, self.file, x[0], None, ] + x[1:]
            self.final_lst[e] = y
        self.final_lst = self.exc_sh + self.final_lst
        return

    def export(self, add=0):
        # if add:
        file2 = f'{lfold}vocab2'
        # else:
        #     file2 = f'{lfold}vocab'
        ef.from_lst2book(self.final_lst, file2)
        ef.open_wb(file2)


def beautify_packhum(file):
    file = f'{dwn_dir}{file}'
    lst = to.from_txt2lst(file, 1)
    for e, x in en(lst):
        x = x.replace('\t', '')
        lst1 = x.split()
        try:
            lst2 = [z for z in lst1 if not reg(r'[0-9]', z)]
            lst2 = [y.replace(';', ',') for y in lst2]
            lst[e] = ' '.join(lst2)

        except:
            p(f'error in {x}')
    lst = elim_end_hyphen(lst)
    to.from_lst2txt(lst, f'{file}2')
    vgf.open_txt_file(f'{file}2')


def beautify_deoque(file):
    file1 = f'{dwn_dir}{file}'
    # pfold = f'{lfold}poetry/'
    # file1 = f"{pfold}{file}"
    file2 = file + '2'
    lst = to.from_txt2lst(file1, 1)
    lst1 = []
    for x in lst:
        if x.startswith('__'):
            lst1.append(x)
        elif '*' in x:
            lst1.append(x)

        elif 'ERROR' in x:
            pass
        elif 'Attenzione!' in x:
            pass

        elif x.isdigit():
            pass
        elif ha(x):
            lst2 = x.split()
            lst3 = [y for y in lst2 if reg(r'[^SD—\|]', y)]
            lst3 = [y.replace(';', ',') for y in lst3]
            s = ' '.join(lst3)
            lst1.append(s)
    lst1 = [x for x in lst1 if hl(x)]
    to.from_lst2txt(lst1, file2)
    vgf.open_txt_file(file2)


def beautify_doc_in_perseus():
    file = 'horace_ap_eng'
    pfold = f'{lfold}poetry/'
    file1 = f"{pfold}{file}"
    file2 = f"{pfold}{file}2"
    lst2 = to.from_xml2txt(file1)
    to.from_lst2txt(lst2, file2)
    vgf.open_txt_file(file2)


'''
6.25 - the models are mostly correct except the amo 
model has two forms one of which removes the "av"



'''


if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'ph', 'lpt', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    if args[1] == 'nw':
        ins = new_words()
        ins.begin(args[2])
    elif args[1] == 'fo':
        focus(args[2])
    elif args[1] == 'er':
        extract_rearrange(args[2])

    elif args[1] == 'nwa':
        # if we are adding more words but using ;
        ins = new_words()
        ins.begin_add(args[2])

    elif args[1] == 'lpt':
        ins = lasla_pos_txt()
        ins.begin()

    elif args[1] == 'pu':
        ins = punctuate_lasla()
        ins.begin()

    elif args[1] == 'nw2':
        '''
        if there are @ on the foreign text
        '''
        ins = new_words()
        ins.begin2(args[2])

    elif args[1] == 'adi':
        '''
        if there are @ on the itrans text
        '''
        ins = new_words()
        ins.begin_add(args[2])

    elif args[1] == 'rt':
        ins = parse_old()
        ins.begin()
    elif args[1] == 'bp':
        beautify_packhum(args[2])
    elif args[1] == 'ph':
        ins = phi()
        ins.begin()

    elif args[1] == 'bd':
        beautify_deoque(args[2])

    elif args[1] == 'rc':
        running_check(args[2])



import sys
import os
import shutil
import urllib.request
import requests
vol = os.path.join(os.path.dirname(__file__)) + '/general_la/'
dir2 = os.path.join(os.path.dirname(__file__))
sys.path.append(dir2)
sys.path.append(vol)

from bglobals import *
import c_colat1
import c_colat2
import c_colat3
import c_colat4
import e_pedocerto
from e_pedocerto import pros, vow_marked
if not public:
    import i_scrape_old
    from i_scrape_old import old_entry
import j_lasla
import j_lasla2
import m_stems
import o_memorize
import v_reading


def elim_useless_files():
    to_del = []
    for l in os.listdir(fold):
        file = f'{fold}{l}'
        if file not in files_used:
            dest = f'{bfold}{l}'
            shutil.copy(file, dest)
            to_del.append(file)
    for x in to_del:
        os.remove(x)
    return




def begin():
    ins = c_colat1.get_co_lemmas1()
    ins.begin()
    ins = c_colat2.bottom_most()
    ins.begin()
    ins.kind = 'p'
    ins.begin(1)
    ins = c_colat3.colatinus()
    ins.begin()
    ins = c_colat4.bottom_most_a4()
    if public:
        ins.begin_fc(0,'rs')
    else:
        ins.begin_fc()
    ins = e_pedocerto.pedecerto()
    ins.begin()
    ins = e_pedocerto.long_by_pos()
    ins.begin3()
    ins = e_pedocerto.check_vowels()
    ins.begin_f()
    if not public:
        ins = i_scrape_old.bottom_most()
        ins.begin()
    ins = j_lasla.convert2txt()
    ins.kind = ''
    ins.begin_ct()
    ins = j_lasla2.bottom_most_la()
    ins.begin_mcl(1)
    ins = m_stems.bottom()
    ins.begin_st()
    ins = o_memorize.bottom_most()
    ins.begin('all', 'start')
    ins = v_reading.punctuate_lasla()
    ins.begin()


begin()
first = 0
if first:
    elim_useless_files()



public = 1
'''
make sure public is set to 1, when I code for myself
the general functions folder is used for all of my separate
projects, but since that would confuse other people only
those functions relevant to this project are used.
'''


from bglobals import *


def help():
    p('''
    
    the purpose of this model is to find the roots of latin words
    since if you know the roots you can easily infer the forms of
    child words.
    we divide words into the following categories:
    
    alternates: alternate spellings
    
    bachelors: words to which no suffix or prefix is ever added
        and are not compound
    
    parents: words which do not contain a prefix or suffix but prefixed
        and suffixes are added to them
    
    children: words which contain a parent
    
    we now divid the children into categories:
    orphans: words composed of prefixes and/or suffixes but the
        parent nolonger exists in the language.  each orphan
        however must be similar in meaning to its sibling orphans
    
    we perform this in broad strokes as follows.
    1) there are 4 text files where we manually went through and weeded
    out children from non-children.  these text files are: stem_lst, check_doubles
    check_reals, check_fakes.  How the files are parsed is explained on
    the file themselves
    
    2) we then put each child into a dict called word2morph whose key is the word
        and whose value is the divisions of this word
    
    3) we then use the technique of cutting off prefixes and a few
        very famous words which often end words such as facio and dico
    
    4) we then divide the potential children into two categories, real and fake
        the reals contain morphemes which we know to be bonafide children,
        the fakes do not.  the fakes are put into the method
        temp_word2morph
    
    we have not yet formed this step but once the oxford latin dictionary
        is fully parsed we will go through the and weed out the
        children from the non-children
        
    
    
    
    
    
    ''')




class top:
    def __init__(self):
        pass

    def get_atts(self):
        self.not_compound = set()
        self.not_compound2 = set()
        self.popular = set()
        self.not_stem = set()
        self.umorphemes = defaultdict(set)
        self.word2morph = defaultdict(set)
        self.failed_fakes = set()
        self.all_stems = set()
        self.no_final = {}
        self.dubious = set()
        self.double2prefix = {}
        self.isalt = {}
        self.not_morpheme = set()
        self.dimin = {}
        # self.def_lemmas = pi.open_pickle(f'{fold}def_lemmas', 1)
        self.co_lemmas4 = pi.open_pickle(f'{fold}co_lemmas4', 1)
        # self.mmodels = pi.open_pickle(f'{fold}mod_vow2', 1)
        self.mmodels = pi.open_pickle(f'{fold}mmodels', 1)
        self.mod2word = pi.open_pickle(f'{fold}mod2word', 1)
        self.lem_freq_crude = pi.open_pickle(f'{fold}lem_freq_crude')
        self.declinable = {}
        self.alt_morph = {}
        self.manual = {}
        self.initials = defaultdict(set)
        self.endings2 = defaultdict(list)
        lst = to.from_txt2lst(f'{fold}check_reals', 1, 1)
        lst1 = to.from_txt2lst(f'{fold}check_fakes', 1, 1)
        self.real_check = lst + lst1
        self.irr_notes = {}
        self.isproper = set()
        self.ie_root = []
        self.non_lemma = set()
        self.word2end = {}
        self.iwords = set()
        self.owords = set()
        self.bachelors = set()
        self.true_fake_stem = set()
        self.temp_word2morph = defaultdict(set)
        self.suffixes = {'faci', 'fact', 'fic', 'fici', 'dic'}

    def cut_off_suffix(self, word, suff):
        if suff == 0:
            l = self.word2end.get(word)
            if not l:
                return word, ''
            return l[0], l[1]
        elif not suff:
            return word

        return word[:-len(suff)]

    def get_suffix(self, word, beg):
        return word[len(beg):]

    def cut_off_prefix(self, word, pre):
        return word[len(pre):]



    def model_by_end(self):
        ends = set()
        for k, v in self.mmodels.items():
            if k == 'miles':
                bb = 8
            for num, lsts in v['des'].items():
                if num == 12:
                    bb = 8
                for l in lsts:
                    for z in l[1]:
                        if z:
                            z = unidecode(z)
                            z = re.sub(r'[0-9]', "", z)
                            ends.add(z)

            if k == "clio_g":
                break
        self.end2 = ends
        return






class use_prefixes(top):
    def __init__(self):
        top.__init__(self)

    def begin_up(self):
        # self.longest_words()
        self.get_pref()
        self.fix_double_cons_prefix()
        self.get_more_real_stems()
        self.rank_lemmas()
        self.popular_words()
        self.get_failed_fakes()
        self.check_spell()
        self.del_ambig_split()
        self.research_failed_fakes()

    def chop_stem(self, w):
        if w.endswith('ior'):
            w = w[:-3]
        elif w.endswith('or'):
            w = w[:-2]
        elif w in ['pesco']:
            w = w[:-1]

        elif reg(r'(e|i)sco$', w):
            w = w[:-4]
        else:
            w = w[:-1]

        if len(w) < 1:
            bb = 8

        return w

    def detach_prefix(self, word, prefix, stem, end):
        c = 0
        if len(word) > len(prefix) + 1 and prefix in self.pref2 \
                and word[len(prefix)] == word[len(prefix) + 1]:
            c = 1
            prefix = prefix + word[len(prefix) + 1]
            stem = stem[len(prefix):]

        else:
            stem = self.cut_off_prefix(stem, prefix)

        if prefix == 'o' and not word[:2] == 'ob' and not c:
            self.owords.add(word)

        if prefix == 'i':
            self.iwords.add(word)
            if len(word) < 3:
                return 0
            elif word[1] == 'g':
                pass
            elif word[1] == word[2] and word[2] in 'lr':
                pass
            else:
                return 0

        if stem in self.not_stem:
            return 0

        if stem in self.true_fake_stem or \
                stem + end in self.popular:
            self.temp_real_stem = (prefix, stem)
        else:
            self.temp_fake_stem = (prefix, stem)

        return 1

    # def longest_words(self):
    #     dct = {}
    #     for x, y in self.stem2lemma.items():
    #         dct[x] = len(x)
    #     self.longest = sort_dct_val_rev(dct)
    #     self.stem2lemma = {k: self.stem2lemma[k] for k, v in self.longest.items()}

    def get_pref(self):
        self.pref, self.pref2 = prefixes()
        self.pref2.append('i')
        self.pref['su'] = 'su'
        self.pref['o'] = 'o'
        self.pref['i'] = 'i'

    def fix_double_cons_prefix(self):
        '''
        here we fix the problem where in praecolligo, for example,
        with ll one l is in the prefix the other in the stem
        '''

        to_de = []
        to_add = {}
        for k, v in self.double2prefix.items():
            if v[1][0] == v[1][1] and (v[0][-1] in self.pref2 or \
                                       v[0][-2:] in self.pref2):

                pre = v[0] + v[1][0]
                stem = v[1][1:]
                _, end = self.word2end.get(k, (k, ''))
                stem = self.cut_off_suffix(stem, end)
                to_de.append(k)
                to_add[k] = (pre, stem, end)
            else:
                _, end = self.word2end.get(k, (k, ''))
                stem = v[1]
                pre = v[0]
                stem = self.cut_off_suffix(stem, end)
                self.double2prefix[k] = (pre, stem, end)
                # p(pre, stem, end)

        for x in to_de:
            del self.double2prefix[x]
        self.double2prefix = merge_2dicts(self.double2prefix, to_add)
        return


    def conditions(self, x):
        if x in self.not_compound:
            return 0
        if x in self.dimin:
            return 0
        if x in self.double2prefix:
            return 0
        if x in self.word2morph:
            return 0
        return 1

    def ad_hoc(self):
        self.word2end['redux'] = ['redux', '']
        self.word2end['res'] = ['res', '']

    def get_failed_fakes(self):
        b = 0
        self.ad_hoc()
        self.all_stems = set(y[0] for x, y in self.word2end.items())
        for x in self.declinable.keys():
            if self.conditions(x):
                if x == 'paratio':
                    bb = 8

                s = x
                stem, end = self.word2end.get(x, (None, None))
                if not stem:
                    # todo all words should have an ending
                    pass
                else:
                    b += 1
                    if b == 38:
                        bb = 8
                    vgf.print_intervals(b, 50)

                    found = 0
                    for z in self.suffixes:
                        if stem.endswith(z) and stem != z:
                            found = 1
                            pref = self.cut_off_suffix(stem, z)
                            stem = self.cut_off_prefix(stem, pref)
                            self.temp_real_stem = (pref, stem)
                            break

                    if not found:
                        self.temp_real_stem = ''
                        self.temp_fake_stem = ''
                        tpref = ""
                        for z in self.pref:
                            if s.startswith(z) and stem > z:
                                found = self.detach_prefix(s, z, stem, end)
                                if found:
                                    tpref += z
                                    break

                    if not found:
                        self.bachelors.add(x)
                    else:
                        self.add2dcts(end, x)

        return

    def check_spell(self):
        dct = {}
        for k, v in self.word2morph.items():
            for tpl in v:
                tpl = [re.sub(r'[0-9]', '', x) for x in tpl]
                s = ''.join(tpl)
                if s != k:
                    dct[k] = v
        if dct:
            assert 0
        return

    def del_ambig_split(self):
        '''
        our code is set up such that a word can be split in different
        ways, ultimately there can only be one way
        '''
        dct = {}
        b = 0
        for k, v in self.word2morph.items():
            if len(v) > 1:
                b += 1
                obj = self.manual.get(k)
                if obj:
                    self.word2morph[k] = obj
                else:
                    dct[k] = v
            else:
                tpl = list(v)[0]
                self.word2morph[k] = tpl
        if dct:
            assert 0

    def get_more_real_stems(self):
        for k, v in self.manual.items():
            vu = self.parse_morpheme(v[1])
            if vu not in self.pref:
                self.true_fake_stem.add(v[1])
        for k, v in self.double2prefix.items():
            vu = self.parse_morpheme(v[1])
            if vu not in self.pref:
                self.true_fake_stem.add(v[1])
        for k, v in self.word2end.items():
            vu = self.parse_morpheme(v[1])
            if vu not in self.pref and k not in self.dimin:
                self.true_fake_stem.add(v[1])

    def parse_morpheme(self, w):
        return re.sub(r'0-9', '', w)


    def rank_lemmas(self):
        dct = sort_dct_val_rev(self.lem_freq_crude)
        b = 0
        dct1 = {}
        for k,v in dct.items():
            dct1[k] = b
            b += 1
        for k,v in self.declinable.items():
            ku = cut_num(k)
            v['rank'] = dct1.get(ku,99_000)


    def popular_words(self):
        for k, v in self.declinable.items():
            if v['rank'] < 8_000:
                self.popular.add(norm_str_jv(v['lemma']))

    def large_endings(self):
        dct = defaultdict(dict)
        for i in range(2, 5):
            dct1 = defaultdict(set)
            dct2 = defaultdict(int)
            for k, v in self.word2end.items():
                s = v[0]
                if len(s) > i + 1:
                    suff = s[-i:]
                    dct1[suff].add(k)
                    dct2[suff] += 1
            dct2 = sort_dct_val_rev(dct2)
            dct3 = {}
            for k in dct2.keys():
                dct3[k] = dct1[k]
            dct[i] = dct3

    def research_failed_fakes(self):
        # 11711 - fake 17638
        # 12559 - fake 16790
        # fake stems - 2481, real words = 12559
        # fake stems - 1497, real words = 18040
        error = {}
        dct = defaultdict(int)
        for k, v in self.temp_word2morph.items():
            v = list(v)[0]
            mid = v[1]
            dct[mid] += 1
            self.temp_word2morph[k] = v

        c = 0
        for k, v in dct.items():
            if v > 1 and reg(r'[aeiouy]', k):
                c += 1
        p(c, len(self.word2morph))

        for k, v in self.temp_word2morph.items():
            mid = v[1]
            if dct[mid] > 1:
                self.word2morph[k] = v
            else:
                self.failed_fakes.add(k)
                self.bachelors.add(k)

        return


    def add2dcts(self, end, word):
        if self.temp_real_stem:
            tpref = self.temp_real_stem[0]
            stem = self.temp_real_stem[1]
            self.word2morph[word].add((tpref, stem, end))

        elif self.temp_fake_stem:
            tpref = self.temp_fake_stem[0]
            stem = self.temp_fake_stem[1]
            self.temp_word2morph[word].add((tpref, stem, end))
        if tpref + stem + end != word:
            bb = 8

    def get_non_stems(self):
        '''
        this is an ad hoc list of stems and non-stems,
        i forget where it came from but currently it works
        as a good filter
        '''

        lst = to.from_txt2lst_tab_delim(f'{fold}stems', 1, 1)
        for l in lst:
            if l[0][0] == ';':
                self.not_stem.add(l[0][1:])
            elif l[0][0] == '#':
                s = l[0][1:]
                l2 = vgf.strip_n_split(s, '+')
                x = l2[0]
                y = l2[1]
                if x in self.declinable:
                    self.isalt[y] = x
                elif y in self.declinable:
                    self.isalt[x] = y
            elif len(l) > 1:
                w = l[0]
                w = self.chop_stem(w)
                self.true_fake_stem.add(w)
            else:
                w = l[0]
                w = self.chop_stem(w)
                self.true_fake_stem.add(w)

        return


class process_stem_lst(top):
    def __init__(self):
        top.__init__(self)

    def main_loop(self, kind='', lst=[]):
        lst1 = []
        if kind == 's':
            lst = to.from_txt2lst(f'{fold}stem_lst', 1, 1)
            lst1 = to.from_txt2lst(f'{fold}manual_stems')
            lst += lst1
        elif kind == 'd':
            lst = to.from_txt2lst(f'{fold}check_doubles', 1, 1)

        self.pre = 1
        self.on = 0
        self.kind = kind
        self.strong = 0
        self.medium = 0
        self.word = ''
        self.is_stem = 0
        self.root_wo_num = ''
        self.no_end = set()

        '''
        removing the * at the end of a word since it does't mean
        anything and is hard to code
        '''
        for e, x in en(lst):
            if '* -- ' in x:
                idx = x.index('*')
                lst[e] = delete_at_i(idx, x)

        for e, x in en(lst):
            if 'paratio' in x:
                bb = 8
            self.end2 = ''

            if x == 'zzz':
                '''
                on the stem_lst sheet the roots before zzz the words end with the root
                after zzz the words start with the root
                '''
                self.pre = 0

            elif x[0] == '/':
                pass
            elif x in lst1:
                # these are the ad hoc manual divisions
                self.word = x
                self.use_slash()

            elif x.isdigit():
                pass

            elif self.kind == 'r' and x.startswith('__') \
                    and x.endswith('#'):
                self.on = 0
                self.not_morpheme.add(x[2:-1])

            elif x.startswith('__'):
                # if it goes in here then it is a root
                if x.endswith('$'):
                    x = x[:-1]
                elif x.endswith('%'):
                    x = x[:-1]
                    self.ie_root.append(x[2:-1])

                elif x.endswith('__') and not self.kind == 'd':
                    x = x[:-2]

                elif x.endswith(')'):
                    idx = x.index(' ')
                    ot = x[idx:].strip()
                    x = x[:idx]
                    self.irr_notes[x[2:]] = ot

                if '=' in x:
                    lst2 = x.split('=')
                    self.alt_morph[lst2[0][2:]] = lst2[1]
                    x = lst2[0]

                self.on = 1
                if not self.is_stem and e > 1 and not self.kind == 'd':
                    self.not_stem.add(self.root.replace('__', ''))

                self.is_stem = 0
                x = x[2:]
                self.strong = 0
                self.medium = 0
                if x.endswith('*'):
                    self.strong = 1
                    x = x[:-1]

                elif x.endswith('>'):
                    x = x[:-1]
                    self.medium = 1
                    self.dubious.add(x)

                elif not self.kind == 'r':
                    self.on = 0
                    self.not_stem.add(x.replace('__', ''))  # 382
                if self.kind == 'd':
                    x = x[:-2]

                self.root = x
                self.root_wo_num = self.root
                if self.root[-1].isdigit():
                    self.root_wo_num = self.root[:-1]


            elif x[-1] == '@':
                # below here we handle words
                self.word = x[:x.index(' ')]
                self.non_lemma.add(self.word)

            elif x[-1] == '#':
                self.word = x[:x.index(' ')]
                self.not_compound.add(self.word)

            elif reg(r'^[a-z//]*\s--', x):
                self.word = x[:x.index(' ')]
                if self.word == 'credulus':
                    bb = 8

                if x.endswith('cc'):
                    self.isproper.add(self.word)  # 1

                elif not self.conditions_psl(self.word):
                    pass

                elif not self.on and self.kind == 'r':
                    self.not_compound.add(self.word)


                elif self.kind == 'd' and (x[-1] == '$' or self.strong):
                    suff = self.word[len(self.root):]
                    self.double2prefix[self.word] = (self.root, suff)

                elif '/' in self.word:
                    self.use_slash()

                elif not self.on and '/' not in self.word:
                    pass
                else:
                    self.is_stem = 1
                    self.num = ''
                    self.tstrong = -1
                    if x[-1].isdigit():
                        self.num = x[-1]
                    elif x[-1] == ')':
                        pass
                    elif x[-1] == '*':
                        self.tstrong = 1
                    elif x[-1] == '>':
                        self.tstrong = 0
                        if x[-2].isdigit():
                            self.num = x[-2]

                    if x[-1] == '?':
                        # these are words that we're
                        # unsure of if they contain a morpheme
                        # i think at the moment nothing is going in here
                        self.add2umorphemes()
                    else:
                        self.divide_word()


        lst = ['no_end', 'root_wo_num']
        for x in lst:
            delattr(self, x)
        return

    def conditions_psl(self, w):
        w = w.replace('/', '')
        if w in self.dimin:
            return 0
        if w in self.isalt:
            return 0
        if w not in self.declinable:
            return 0
        return 1

    def get_end(self):
        self.end2 = self.word2end.get(self.word)
        if self.end2 == None:
            self.no_end.add(self.word)
            self.end2 = ''
        else:
            self.end2 = self.end2[1]




    def use_slash(self):
        '''
        if self.medium = 1 then the final is a mere suffix
        whereas if self.strong = 1 then the final is a stand alone word

        '''

        triple = 0
        mid = ''
        lsuf = ''
        quad = 0
        lst = self.word.split('/')
        if self.word.count('/')>2:
            quad = 1

        elif self.word.count('/') > 1:
            triple = 1
        self.initial = f'{lst[0]}'

        if quad:
            self.initial = lst[0]
            mid = f'{lst[1]}'
            self.final = f'{lst[2]}'
            self.end2 = f'{lst[3]}'

        elif triple:
            mid = f'{lst[1]}'
            self.final = f'{lst[2]}'
        else:
            self.final = f'{lst[1]}'

        self.word = self.word.replace('/', '')
        if not quad:
            self.get_end()
            if self.end2:
                self.final = self.cut_off_suffix(self.final, self.end2)

        if quad:
            self.word2morph[self.word].add((self.initial, mid, self.final, self.end2))

        elif triple:
            self.word2morph[self.word].add((self.initial, mid, self.final, self.end2))
        else:
            self.word2morph[self.word].add((self.initial, self.final, self.end2))

        if self.medium:
            self.endings2[self.final[1:]].append(self.word)
        self.add2manual(mid)


    def add2manual(self, mid):
        if mid:
            self.manual[self.word] = (self.initial, mid, self.final, self.end2)
        else:
            self.manual[self.word] = (self.initial, self.final, self.end2)


    def add2umorphemes(self):
        '''
        these are morphemes that i'm not very certain of
        '''
        pass
        # self.umorphemes[self.initial + "-"].add(self.word)

    def divide_word(self):
        '''
        some very bad code.  the problem with this method is that i did
        not have a system going when i built the sheets stem_lst, check_reals etc
        if the word begins with a root then it is relatively easy to divide the word
        but if the word does not begin with a root then it is quite hard
        in some cases the root contains the ending of the word
        in other cases there are some letters between the root and the ending
        in other cases the root + the ending compose the end of the word
        '''

        self.get_end()
        lsuf = ''
        if self.word.startswith(self.root_wo_num):
            rest = self.get_suffix(self.word, self.root_wo_num)
            final2 = self.cut_off_suffix(rest, self.end2)
            final = final2
            initial = self.root_wo_num
            initial2 = self.root
        else:
            initial = self.word[:self.word.index(self.root_wo_num)]
            initial2 = initial
            s = self.root_wo_num + self.end2
            su = initial + s

            if su == self.word:
                # in this case the word is already properly divided
                final = self.root_wo_num
            else:
                if len(su) > len(self.word):
                    # here the root and the ending overlap
                    mid = self.word[len(initial):-len(self.end2)]
                    lsuf = self.get_suffix(mid, self.root_wo_num)
                    final = self.root_wo_num

                    v = initial + mid + lsuf
                    if v == self.word:
                        # here the ending was not obtained
                        vnew = self.cut_off_suffix(v, self.end2)
                        if vnew == self.end2:
                            # here the root + lsuf = end of the word
                            if not lsuf:
                                if self.num:
                                    assert 0
                                else:
                                    final = self.cut_off_suffix(self.root_wo_num, self.end2)
                            else:
                                assert 0
                        else:
                            if vnew == self.root_wo_num:
                                final = self.root_wo_num
                            else:
                                lsuf = self.get_suffix(vnew, self.root_wo_num)

                    else:
                        if initial + mid + lsuf + self.end2 == self.word:
                            final = mid
                        else:
                            assert 0
                else:
                    if not self.end2 and not self.word.endswith(self.root_wo_num):
                        # here the ending is not obtained and the word does not end with the root
                        v = initial + self.root_wo_num
                        lsuf = self.get_suffix(self.word, v)
                        final = self.root_wo_num
                    else:
                        # here there are some letters between the root and the ending
                        mid = self.word[len(initial):-len(self.end2)]
                        lsuf = self.get_suffix(mid, self.root_wo_num)
                        final = self.root_wo_num

            final2 = final + self.num

        if not final:
            self.no_final[self.word] = (initial, self.end2)
            return

        if self.word != f"{initial}{final}{lsuf}{self.end2}":
            bb = 8

        self.word2morph[self.word].add((initial2, final2, lsuf, self.end2))



class stems(process_stem_lst, use_prefixes):
    def __init__(self):
        process_stem_lst.__init__(self)
        use_prefixes.__init__(self)

    def change_spelling(self):
        '''
        this changes the macrons on a few words but this step will
        soon be obsolete anyway since we will soon use more
        sophisticated methods to determine vowel length

        '''

        self.spell_change = {}
        lst = to.from_txt2lst_tab_delim(f'{fold}mispellings', 1, 1)
        for l in lst:
            self.spell_change[l[0]] = l[1]

        for x, y in self.spell_change.items():
            self.co_lemmas4[x]['lemma'] = y

        # for x, ins in self.def_lemmas.items():
        #     if ins.pos == 'v' and ins.macron[-1] == 'o':
        #         ins.macron = f'{ins.macron[:-1]}ō'
        #     elif reg(r'o\d$', ins.macron):
        #         ins.macron = replace_at_i(len(ins.macron) - 2, ins.macron, 'ō')

        return

    def get_declinable(self):
        '''
        we are now only interested in finding the roots of
        declinable words, nouns, verbs and adjectives, ignoring adverbs for now.
        we also ignore some words that belong to specific modules
        '''

        for x, y in self.co_lemmas4.items():
            if y['model'] in ['eo', 'fio', 'do', 'aio', 'duo', 'deni']:
                pass
            elif y['pos'] in ['n', 'v', 'a'] and not y['capital']:
                self.declinable[x] = y



    def cut_off_ending(self):
        '''
        we here separate the ending from the rest of the word.
        we are not here concerned with large endings such as -mentum
            in the word argumentum, rather just the um part.
        the verbs are rather straightforward.

        some nouns are a little hard to deal with.  for most we
        take the latters in the nominative singular which undergo
        change in the other declensions.  for some nouns this does
        not work. in this case we take the number in des key in the
        model dictionary, the first member and subtract one from it,
        that is how many letters we cut off at the end.  this algorithm
        works find for

        our algorithm works fine for: pulcher, corpus, animal, miser
            miles_ium+, puer, infans

        for reasons unknown to me, the number for the following models was wrong though
            they still decline correctly

            for facilis, fortis, civis_um, civis3, civis2, civis4, acer, miles_s the last 2 letters are cut off

        for those we substract the last two letters.

        for miles we had to find the stem in an ad hoc manner

        there are a few words in civis where the final letters can be
        cut off but not many.  ignore these for now
        todo find stems in the civis model

        '''

        dct = defaultdict(list)
        for k, v in self.co_lemmas4.items():
            if k.startswith('maximus2'):
                bb = 8

            if not v['capital'] and v['pos'] in ['a', 'v', 'n']\
                    and v['model'] not in ['prcum','se']:
                mod = self.mmodels[v['model']]
                w = cut_num(k)
                if v['pos'] == 'v':
                    found = 1
                    l2 = w[-2:]
                    l1 = w[-1]
                    if l2 == 'or':
                        stem = w[:-2]
                        end = 'or'
                    elif l1 == 't':
                        stem = w[:-2]
                        end = w[-2:]
                    elif l1 in ['o', 'i']:
                        stem = w[:-1]
                        end = l1

                    elif l1 == 'm':
                        stem = w
                        end = ""
                    else:
                        '''only a few haplaxes go in here'''
                        found = 0

                else:
                    if '_pl' in v['model'] or v['model'] in ['nos','uos']:
                        lst = mod['des'][7]
                    elif v['pos'] == 'a':
                        lst = mod['des'][13]
                    else:
                        lst = mod['des'][1]
                    found = 0
                    should = 0
                    for t in lst:
                        for x in t[1]:
                            if x:
                                should = 1
                                xu = unidecode(x)
                                if v['lemma'].endswith(xu):
                                    stem = w[:-len(xu)]
                                    end = xu
                                    found = 1
                                    break
                            else:
                                found = 1
                                if v['model'] == 'miles':
                                    stem, end = self.stem_miles(w)

                                elif t[0] == '0':
                                    stem = w
                                    end = ''
                                elif v['model'] in ['uetus', 'plus', 'miles_g', 'civis']:
                                    stem = w
                                    end = ''


                                elif v['model'] in ['facilis', 'fortis', 'civis_um', 'civis3', 'civis2', 'civis4', 'acer',
                                                 'miles_s']:
                                    stem = w[:-2]
                                    end = w[-2:]
                                else:
                                    stem = w[:-(int(t[0]) - 1)]
                                    end = w[-(int(t[0]) - 1):]
                                break

                        if not found and should:
                            dct[v['model']].append(v)
                if found:
                    if end and not v['lemma'].endswith(end):
                        bb = 8

                    self.word2end[w] = [stem, end]

        return

    def stem_miles(self, w):
        try:
            l3 = w[-3:]
        except:
            l3 = ''
        try:
            l2 = w[-2:]
        except:
            l2 = ""
        try:
            l4 = w[-4:]
        except:
            l4 = ""

        if l4 in ['tudo', 'trix']:
            return w[:-4], l4

        if l3 in ['tio', 'tor', 'tas', 'fex']:
            return w[:-3], l3

        if l2 in ['io', 'or']:
            return w[:-2], l2

        return w, ""

    def elim_diminutives1(self):
        '''
        todo code for ittus
        -lus
            -ulus, -ula, -ulum, e.g. globulus (globule) from globus (globe).
    -culus, -cula, -culum, e.g. homunculus (so-small man) from homo (man)
    -olus, -ola, -olum, e.g. malleolus (small hammer) from malleus (hammer)
    -ellus, -ella, -ellum, e.g. libellus (little book) smaller than librulus (small book) from liber (book)
    -ittus, -itta, -ittum (hypocoristic, a doublet of -itus)
        '''

        dct = {
            'uncula': ['o'],
            'unculus': ['o'],
            'ellus': ['er', 'a', 'us', 'ulus'],
            'ellum': ['rum'],
            'ella': ['a'],
            'icula': ['us', 'a'],
            'ulum': ['um', 'a', 'e'],
            'olus': ['us'],
            'olum': ['um'],
            'ola': ['a'],
            'ole': ['um'],
            'ulus': ['us', 'a', 'e', 'o'],
            'ula': ['a', 'e'],
        }
        dct1 = {
            'uncula': 'a',
            'unculus': 'us',
            'ellus': 'us',
            'ellum': 'um',
            'ella': 'a',
            'icula': 'a',
            'ulum': 'um',
            'olus': 'us',
            'olum': 'um',
            'ola': 'a',
            'ole': 'e',
            'ulus': 'us',
            'ula': 'a',
        }

        self.dimin = {}
        for k, v in self.declinable.items():
            found2 = 0
            for x, y in dct.items():
                if found2:
                    break
                if k.endswith(x):
                    for t in y:
                        ku = k[:-len(x)] + t
                        if ku in self.co_lemmas4:
                            if k == 'naulum':
                                bb = 8

                            found2 = 1
                            initial = self.cut_off_suffix(k, x)
                            end = dct1[x]
                            lsuf = self.cut_off_suffix(x, end)
                            self.word2morph[k].add((initial, lsuf, end))
                            self.dimin[k] = ku
                            break

        return

    def elim_diminutives2(self):
        '''
        todo further split diminutives
        do this later, some diminutives can be further split

        '''

        for k, v in self.dimin.items():
            obj = self.word2morph.get(v)
            parent = self.word2morph.get(v)
            if parent:

                to_add = []


        return




class bottom(stems):
    def __init__(self):
        stems.__init__(self)

    def begin_st(self):
        p ('now dividing words into morphemes')
        '''
        the methods get_endings, ana_homop and permutations might
            still be useful

        '''

        self.kind = 'l'
        self.get_atts()
        self.change_spelling()
        self.get_declinable()
        self.cut_off_ending()
        self.get_non_stems()
        self.elim_diminutives1()
        self.main_loop('d')
        self.main_loop('s')
        self.main_loop('r', self.real_check)
        self.begin_up()
        self.elim_diminutives2()
        self.output()
        return

    def output(self, kind=0):
        '''
        work must still be done on diminutives
        the manual attribute represents words which have
        been already verified
        the not_compound attribute contains words that are not compound
            but of course contain an ending
        the double prefix contains all words  which contain
            a double prefix

        '''
        pi.save_pickle(self.word2morph, f'{fold}word2morph')
        pi.save_pickle(self.double2prefix, f'{fold}double2prefix')





        return


    def contains_prefix(self):
        for k,v in self.word2morph.items():
            pass






if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'me', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    if args[1] == 'me':
        ins = bottom()
        ins.begin_st()
    elif args[1] == 'psl':
        ins = process_stem_lst()
        ins.main_loop()




import pdb, json, re
from operator import itemgetter


p = print

en = enumerate

db = pdb.set_trace
conditional = chr(8594)
ds = ".DS_Store"
ds1 = '._.DS_Store'

# vol = '/users/kylefoley/'
#
# fdir = vol + 'documents/'
# fdir2 = vol + 'documents/'
# mdir = vol + 'documents/codes/'
files_used = set()
# pdir = fdir + 'pcode/'
# base_dir = '/Users/kylefoley/documents/pcode/'
# dwn_dir = '/users/kylefoley/downloads/'

dia = 'àáäâèéêëìíïîòóöôùúûüçßñ'
diau = 'ÀÁÄÂÈÉÊËÌÍÏÎÒÓÖÔÙÚÛÜÇÑ' + dia

open_apost = chr(8220)

closed_apost = chr(8221)

large_dash = chr(8212)

upside_down_q = chr(191)

ellip = chr(8230)

obje = chr(65532)

letter_2numbers = "abcdefghijklmnopqrstuvxyz"

letters = "abcdefghijklmnopqrstuvxyz"

dsstore = '.DS_Store'

not_execute_on_import = '__name__ == "__main__"'

sort_dct_key = lambda x: from_tpl2dct(sort_dct_key_tpl(x))

sort_dct_key_tpl = lambda x: sorted(x.items(), key=itemgetter(0))

ujsonc = lambda x: json.loads(json.dumps(x))

sort_dct_val_rev = lambda x: from_tpl2dct(sort_dct_val_rev_tpl(x))

sort_dct_val = lambda x: from_tpl2dct(sort_dct_val_tpl(x))

sort_by_col_rev = lambda lst, col: sorted(lst, key=itemgetter(col), reverse=True)

sort_dct_val_rev_tpl = lambda x: sorted(x.items(), key=itemgetter(1), reverse=True)

sort_by_col = lambda lst, col: sorted(lst, key=itemgetter(col))

hl = lambda x: bool(re.search(r'\S', x))  # has non-characters

ha = lambda x: bool(re.search(r'[a-zA-Z]', x))  # has alphabetic characters

reg = lambda x, y: bool(re.search(x, y))

sort_dct_val_tpl = lambda x: sorted(x.items(), key=itemgetter(1))

from_tpl2dct = lambda x: {z[0]: z[1] for z in x}

get_between = lambda sent, l, r: sent[sent.index(l) + 1:sent.index(r)]

percent = lambda x, tot: int(round(x / tot, 2) * 100)

add_at_i = lambda idx, sent, char: sent[:idx] + char + sent[idx:]

replace_at_i = lambda idx, sent, char: sent[:idx] + char + sent[idx + 1:]

merge_2dicts = lambda x, y: {**x, **y}

delete_at_i = lambda x, sent: sent[:x] + sent[x + 1:]

jsonc_lam = lambda x: json.loads(json.dumps(x))

def jsonc(obj):
    if type(obj) == dict:
        for x, y in obj.items():
            if type(x) == int:
                try:
                    obj = ujsonc(obj)
                except:
                    obj = jsonc_lam(obj)
                return {int(k): v for k, v in obj.items()}
            else:
                break
    try:
        obj = ujsonc(obj)
    except:
        obj = jsonc_lam(obj)
    return obj


from bglobals import *

lvow = {
    'a': 'ā',
    'e': 'ē',
    'i': 'ī',
    'o': 'ō',
    'u': 'ū',
    'y': 'ȳ'
}
bvow = {
    'a': 'ă',
    'e': 'ĕ',
    'i': 'ĭ',
    'o': 'ŏ',
    'u': 'ŭ',
    'y': 'ў'
}

dctabb = {
    'a-': 'ă',
    'e-': 'ĕ',
    'i-': 'ĭ',
    'o-': 'ŏ',
    'u-': 'ŭ',
    'v-': 'ŭ',
    'y-': 'ў',
    'a+': 'ā',
    'e+': 'ē',
    'i+': 'ī',
    'o+': 'ō',
    'u+': 'ū',
    'v+': 'ū',
    'y+': 'ȳ',
}



class pros:
    def __init__(self, lst, ped=0, ucons_dct={}):
        self.name = lst[0].lower()
        self.raw = lst[1]
        if self.raw == 'qui+nque-iu-go':
            bb = 8

        self.wmac = lst[1]
        self.vowels = lst[2]
        self.tot = int(lst[4])
        self.ratio = int((int(lst[3]) / self.tot) * 100)
        if ped:
            self.macronize(ucons_dct)

    def macronize(self, ucons_dct):
        s = self.wmac
        for x, y in dctabb.items():
            s = s.replace(x, y)

        if len(s) > 2:
            te = jv.replace(unidecode(s))
            idx = ucons_dct.get(te)
            if idx:
                # p (te)
                s = replace_at_i(idx, s, conu)

            lvow = last_vowel(unidecode(s))
            sec = s[lvow:]
            fir = s[:lvow]
            s = self.restore_jv(fir) + sec

        self.wmac = s
        self.jv_no_mac = unidecode(self.wmac)
        self.iu = jv.replace(self.jv_no_mac)
        return

    def restore_jv(self, fir):
        dct2 = {
            'ē': 'e',
            "ū": 'u'
        }
        if not fir:
            return ''
        b = 0
        if fir[0] == 'i':
            fir = replace_at_i(0, fir, 'j')

        for x, y in zip(fir[:-1], fir[1:]):
            xl = x
            for s, t in dct2.items():
                xl = xl.replace(s, t)
            if y == 'i' and xl not in 'eu':
                fir = replace_at_i(b + 1, fir, 'j')
            elif y == 'u' and xl not in 'qae':
                bb = 8
            b += 1
        return fir

    def __repr__(self):
        return self.iu


class pedecerto:
    '''
    this class must be run before macronizer_cl, since the latter
    weeds out its words based on whether or a word exists in
    the prosodic file
    '''

    def __init__(self):
        pass

    def begin(self):
        self.get_atts()
        self.make_class()
        # self.researchj()
        self.output_p()
        # self.weed_out()

    def begin2(self):
        self.get_atts(1)

    def get_atts(self, second=0):
        if not second:
            self.macronizer = pi.open_pickle(f'{fold}macronizer_new', 1)
            self.prosodic = to.from_txt2lst(f'{fold}prosodic')
            self.word2stats = {}
            self.prosodic_stats = defaultdict(list)
            self.prosodic_stats_jv = defaultdict(list)

        else:
            self.macronizer_pc = pi.open_pickle(f'{fold}macronizer_pc', 1)
            self.pros_stats = pi.open_pickle(f'{fold}pros_stats', 1)
        return

    def output_p(self):
        pi.save_pickle(self.prosodic_stats, f'{fold}prosodic_stats_new', 1)



    def make_class(self):
        '''
        in order to use the ucons dict the macronizer has to be run
        first, it replace u with a u consonant, either that
        or we need to build a ucons dict or perhaps infer
        that if u follows q,g,s and it is not marked then
        it is a consonant
        '''

        b = 0
        p (f"""
        now converting the pedocerto database into a list
        of python classes
""")

        for x in self.prosodic:
            lst = vgf.strip_n_split(x, ";")
            ins = pros(lst, 1)
            self.prosodic_stats[ins.iu].append(ins)
            self.prosodic_stats_jv[ins.jv_no_mac].append(ins)
            b += 1
            vgf.print_intervals(b, 5000, None, len(self.prosodic))
            # if b > 10_000:
            #     break
        return

    def researchj(self):
        '''
        there are mistakes in my algorithm, omnia is turning out
        omnja yet omnja has zero hits
        :return:
        '''

        lst = []
        dct = defaultdict(int)
        for k, v in self.prosodic_stats_jv.items():
            dct[v.iu] += 1

        dct2 = {k: v for k, v in dct.items() if v > 1}
        dct3 = {}
        for k, v in self.prosodic_stats_jv.items():
            if v.iu != v.jv_no_mac and dct2.get(v.iu) and dct[v.iu] > 1:
                itm = self.prosodic_stats.get(v.iu)
                if itm:
                    dct3[v.iu] = [v.jv_no_mac, v.ratio, v.tot, v.iu, itm.ratio, itm.tot]

        return

    def weed_out(self):
        dct = {}
        self.ncol = {}
        for x, y in self.word2stats.items():
            if x not in self.macronizer:
                self.ncol[x] = y.tot
        self.ncol = sort_dct_val_rev(self.ncol)
        for x, y in self.macronizer.items():
            if x in self.word2stats:
                dct[x] = y
        self.macronizer = dct
        pi.save_pickle(self.ncol, f"{fold}macron_pc", 1)







class vow_marked:
    def __init__(self, x, split, y):
        self.original = x
        self.marked = y
        self.code = ''
        self.split = split

    def __repr__(self):
        return self.original


class long_by_pos:
    def __init__(self):
        """
        this class does not save any new object but just tests
        to see if our understanding of long by position is
        correct
        """
        pass

    def begin3(self):
        self.get_atts4(2)
        self.get_atts2()
        self.step2()
        self.test()
        self.mf_diph()
        self.output()
        return



    def get_atts4(self, pede=0):
        self.ucons_dct = {}
        if not pede:
            self.lemmes = to.from_txt2lst(f'{fold}lemmes', 1)
        elif pede == 2:
            self.macronizer = pi.open_pickle(f'{fold}macronizer_new', 1)
            self.prosodic_stats = pi.open_pickle(f'{fold}prosodic_stats_new', 1)
        else:
            self.prosodic = to.from_txt2lst(f'{fold}prosodic')
            self.macronizer = pi.open_pickle(f'{fold}macronizer_new', 1)
            self.prosodic_stats = {}
        return

    def output(self):
        st = set(v[0].iu for k, v in self.still_wrong.items())
        to.from_lst2txt(st, f'{fold}col_ped_diff')



    def research_ui(self):
        dct1 = {}
        dct2 = {}
        for x,y in self.prosodic_stats.items():
            if 'ui' in x:
                if reg(r'u\+i[^\+\-]', y.raw) or reg(r'u\+i$', y.raw):

                    dct1[x] = y.raw
                else:
                    dct2[x] = y


    def special_prefixes(self):
        lst, _ = prefixes()
        liquids = 'bcdgpt'
        exp = ['post', 'amb', 'red', 'sed']
        self.sprefix = [x for x in lst if x[-1] in liquids and x not in exp]

    def temp12(self):

        lst7 = [x for x, y in wrong.items() if any(x.startswith(z) for z in self.sprefix)]
        dct4 = {x: wrong[x] for x in lst7}

    def test(self):
        error = {}
        for x, z in self.prosodic_stats.items():
            for y in z:
                coll = self.macronizer.get(y.iu)
                if coll:
                    wmacu = unidecode(y.wmac)
                    st = set()
                    for k, v in coll.items():
                        k1 = unidecode(k)
                        st.add(k1)
                    col2 = vgf.dct_idx(coll)
                    if wmacu not in st:
                        error[(y.wmac, col2)] = [y, coll]
        self.error = error
        return

    def mf_diph(self):
        dct = {}
        dct2 = {}
        still_wrong = {}
        for k, v in self.error.items():
            ped = k[0]
            col = k[1]
            found = 0
            pedu = unidecode(ped)
            for y in ['ui', 'ei']:
                if y in pedu and 'j' in col:
                    cidx = [e for e, w in en(col) if w == 'j']
                    pidx = pedu.index(y)
                    if ped[pidx + 1] in 'īĭ':
                        pass

                    elif pidx + 1 in cidx:
                        if not pedu.count(y) == 1:
                            found = 1
                            dct2[k] = v
                            break
                        else:
                            found = 1
                            dct[k] = v
                            break
            if not found:
                still_wrong[k] = v
        self.still_wrong = still_wrong

        return

    def get_atts2(self):
        self.ncons = 'aeiouyăĕĭŏŭўāēīōūȳh_'
        singles = ['ch', 'ph', 'th', 'qu']
        singles += [x + 'l' for x in 'bcdgpt']
        singles += [x + 'r' for x in 'bcdgpt']
        self.singles = singles
        self.special_prefixes()
        self.wrong = []

    def step2(self):
        self.wrong = []  # should be 58
        self.single = 0
        self.fdiph = []
        self.sdiph = []
        self.gcons = []
        self.hasj = []
        self.jlong = []
        self.jshort = []
        self.gfl_tot = 0
        self.gfl_rat = 0
        b = 0
        p (f"""
        now distinguishing between syllables which are
        long by position and those that are not
""")

        for k, lst in self.prosodic_stats.items():
            b += 1
            # if b > 10_000:
            #     break

            vgf.print_intervals(b, 500,None,len(self.prosodic_stats))
            for self.ins in lst:
                x = self.ins.wmac
                self.is_heavy(x)
                if len(self.wrong) > 100:
                    bb = 8

        return

    def is_heavy(self, x, kind=""):
        xo = x
        self.syllables = []
        for s in self.singles:
            x = x.replace(s, s[0] + '_')
        y = unidecode(x)

        for s in self.singles:
            y = y.replace(s, s[0] + '_')
        idx = len(x)
        if kind == 'p':
            idx = last_vowel(y)
            x = x[:idx]

        i = 0
        while i < idx:
            xl = x[i]
            xlu = unidecode(xl)
            try:
                x0 = x[i - 1]
            except:
                x0 = ""

            try:
                xl1 = x[i + 1]
            except:
                xl1 = ""
            try:
                xl2 = x[i + 2]
            except:
                xl2 = ""
            combo = xl + xl1
            combo2 = xl1 + xl2
            combo2u = unidecode(combo2)
            combou = unidecode(combo)

            if xl == 'u' and x0 == 'g' and self.quantified(xl1):
                if not self.single:
                    self.gcons.append(x)

            elif combou in diph and conu not in combo:
                if self.single:
                    x, i = self.is_diph(x, i, combo, kind)
                else:
                    if self.quantified(xl1):
                        if not self.quantified(xl):
                            if not self.single:
                                self.add2wrong(x, xl)
                                break
                        else:
                            i += 1

                    elif xl in 'ăĕĭŏŭў':
                        if not self.quantified(xl1):
                            if not self.single:
                                self.fdiph.append([self.ins, x, xl])
                                i += 1
                                break
                        else:
                            if not self.single:
                                self.sdiph.append([self.ins, x, xl])
                            i += 1
                    elif self.has_macron(xl):
                        i += 1
                    elif not self.has_macron(xl):
                        if not self.single:
                            self.add2wrong(x, xl)
                            break


                    bb=8


            elif xlu in 'aeiouy':
                if combo2 in ['fl', 'fr']:
                    if not self.single:
                        self.gfl_tot += self.ins.tot
                        if self.islong(xl):
                            self.gfl_rat += int(self.ins.tot * (self.ins.ratio * .01))

                elif xl1 in ['x', 'z']:
                    if self.single:
                        x = self.underscore(x, i)

                    elif not self.islong(xl):
                        self.add2wrong(x, xl)
                        break

                elif len(combo2) > 1 and \
                        not reg(r'[' + self.ncons + r']', combo2u):
                    if self.single:
                        x = self.underscore(x, i)

                    elif not self.islong(xl):
                        self.add2wrong(x, xl)
                        break

            i += 1
        return x, xo

    def is_diph(self, x, i, combo, kind):
        '''
        kind: c
        the words from the macronizer indicate a diphthong by being
        absent of macrons and breves and belonging to one of the six
        two letter combinations
        kind: p
        the words from prosodic indicate diphthongs by marking the
        first letter as long and not marking the second
        here, if it is a diphthong then the first letter is long and
        the second is an _
        '''
        found = 0
        if kind in ['c','a']:
            if not self.quantified(combo[0]) and not self.quantified(combo[1]):
                x = self.make_long(x, i)
                found = 1
        else:
            if self.islong(combo[0]) and not self.quantified(combo[1]):
                found = 1
        if found:
            return self.underscore(x, i + 1), i + 1
        return x, i

    def underscore(self, x, i):
        # if self.kind == 'a':
        #     return replace_at_i(i, x, '+')
        # else:
        return replace_at_i(i, x, '_')

    def add2wrong(self, x, xl):
        self.wrong.append([self.ins, x, xl, self.ins.ratio, self.ins.tot])

    def quantified(self, x):
        if x in 'āēīōūȳ' + 'ăĕĭŏŭў':
            return 1

    def has_macron(self, x):
        if x in 'āēīōūȳ':
            return 1

    def islong(self, x):
        if x == chr(1263):
            bb = 8

        if x in 'āēīōūȳ' + chr(1263):
            return 1
        return 0

    def isshort(self, x):
        if x in 'ăĕĭŏŭў':
            return 1

    def make_long(self, x, i):
        uvow = unidecode(x[i])
        if not uvow in 'aeiouy':
            return x
        l = lvow[uvow]
        return replace_at_i(i, x, l)

    def make_short(self, x, i):
        uvow = x[i]
        l = bvow[uvow]
        return replace_at_i(i, x, l)


class check_vowels(long_by_pos):
    '''
    the following determines to what extent authors agree
    on the length of vowels

    '''

    def __init__(self):
        long_by_pos.__init__(self)

    def begin(self):
        self.get_atts()
        self.get_atts2()
        self.unambig_words()
        self.mark_long()
        self.evaluate()
        return

    def begin_e(self):
        self.get_atts()
        self.get_atts2()


        self.unambig_words()
        self.mark_long(1)
        #self.by_author() currently has bugs
        return



    def begin_f(self):
        '''
        for reasons that utterly escape i cannot get the obj
        self.prosodic_stats to open a second time.  for this
        reasons i've had to combine long_by_pos.begin3 with
        check_vowels.begin_f

        '''

        self.get_atts4(2)
        self.get_atts2()
        self.get_atts()
        self.step2()
        self.test()
        self.mf_diph()
        self.output()
        self.unambig_words()
        self.mark_long(1)




    def begin_ac(self):
        self.get_atts2()
        self.kind = 'a'
        self.single = 1
        self.lem2forms_jv = pi.open_pickle(f'{fold}lem2forms')
        # self.test_ac()
        self.get_accent()


    def get_atts(self):
        # self.macronizer = pi.open_pickle(f'{fold}macronizer_new', 1)
        self.splits = pi.open_pickle(f'{fold}splits', 1)
        # self.prosodic_stats = pi.open_pickle(f'{fold}prosodic_stats', 1)
        # self.col_ed_diff = set(to.from_txt2lst(f'{fold}col_ped_diff'))

    def unambig_words(self):
        '''
        the following divides words into those which only belong
        to one lemma 'unambig' and those that don't 'ambig'
        those that are unambiguous can be used to determine
        correctness of vowel quantity

        '''

        p (f'''
        now determining which words have no ambiguities
        in their vowel length
''')

        self.unambig = {}
        self.ambig = {}
        errs = defaultdict(list)
        b = 0
        for k, v in self.macronizer.items():
            if len(v) == 1:
                word = vgf.dct_idx(v)
                tpl = self.splits.get(word)
                if tpl:
                    self.unambig[k] = tpl[0]
                else:
                    errs[k].append(word)
            else:
                st = set()
                for x, y in v.items():
                    sp = self.splits.get(x)
                    if sp:
                        st.add(sp[0])
                    else:
                        errs[k].append(x)
                if len(st) == 1:
                    self.unambig[k] = list(st)[0]
                else:
                    self.ambig[k] = v
            b += 1
            vgf.print_intervals(b,20_000,None,len(self.macronizer))

        return

    def by_author(self):
        '''
        the following determines the extent to which each author's
        vowel lengths aggree with the pedecerto database
        '''

        self.macronizer_each = pi.open_pickle(f'{fold}macronizer_each', 1)
        self.single = 1
        auth2wrong = {}
        a2perc = {}
        p (f"""
        we now use the pedecerto database to determine if
        a dictionary author has correctly indicated vowel length
""")

        for auth, dct in self.macronizer_each.items():
            b = 0
            p (f"""
            now evaluating the extent to which the following author: 
            {auth}
            agrees with the others in terms of syllable length
            """)
            self.unambig2 = {}
            for k, v in dct.items():
                x = self.unambig.get(k)
                b += 1
                if x:
                    kind = 'c'
                    marked, orig = self.is_heavy(x, kind)
                    vgf.print_intervals(b, 1000,None,len(dct))
                    v = vow_marked(k, x, marked)
                    code, locs = self.get_code(marked, kind)
                    v.nature = code
                    self.unambig2[k] = v

            auth2wrong[auth], perc = self.evaluate()
            a2perc[auth] = perc
            p (f'{auth} has a success ratio of {perc}')

        return



    def mark_long(self, only_pro=0, lst2=[]):
        self.single = 1
        self.single = 0 # usually 1
        if lst2:
            lst = [self.prosodic_stats, self.unambig]
        else:
            lst = lst2

        self.unambig2 = {}
        p(f"""
        now seeing if we correctly understand how
        pedecerto determines a long syllable 
        """)

        for e, dct in en(lst):
            b = 0
            if e> 0:
                if only_pro and e:
                    break
                for k, v in dct.items():
                    if e == 0:
                        x = v.wmac
                        kind = 'p'
                    else:
                        x = v # v is the first half of a split
                        kind = 'c'
                    marked, orig = self.is_heavy(x, kind) # marked has only the short and long vowels, string
                    if marked != orig:
                        bb=8
                    b += 1
                    vgf.print_intervals(b, 1000, None, len(dct))
                    if e == 0:
                        v.marked = marked
                    else:
                        v = vow_marked(k, v, marked) # k is an unmacronized full word string
                    code, locs = self.get_code(marked, kind)
                    v.nature = code
                    if e:
                        self.unambig2[k] = v
        return

    def test_ac(self):
        self.get_accent2( 'achaeīssimum')


    def get_accent(self):
        for k,v in self.lem2forms_jv.items():
            for pos,y in v.items():

                lst = []
                for z in y[0]:
                    word = self.get_accent2(z)
                    lst.append(word)
                y[0] = lst
                done = 1
                v[pos] = y
        return


    def get_accent2(self, x):
        marked, orig = self.is_heavy(x,'a')  # marked has only the short and long vowels, string
        code, locs = self.get_code(marked,'a')
        if len(code)>2:
            if code[-2] == '+':
                sloc = locs[-2]+1
            else:
                sloc = locs[-3]+1
            x = add_at_i(sloc,x,ac)

        return x




    def get_code(self, x, kind):
        code = ""
        locs = []
        for e,y in en(x):
            if y == '+':
                code += '+'
                locs.append(e)
            elif self.isshort(y) and kind == 'p':
                code += '-'
                locs.append(e)
            elif kind in ['c','a'] and y in 'aeiouy':
                code += '-'
                locs.append(e)
            elif self.islong(y):
                code += '+'
                locs.append(e)

        return code, locs

    def evaluate(self):
        tot = 0
        right = 0
        wrong = {}
        rightd = {}
        for x, y in self.prosodic_stats.items():
            if y.ratio == 100 and y.iu not in self.col_ed_diff:
                ins = self.unambig2.get(x)
                if ins:
                    self.temp_conu(y, ins)
                    tot += 1
                    if len(y.nature) < len(ins.nature):
                        shorter = y.nature
                        longer = ins.nature
                    else:
                        shorter = ins.nature
                        longer = y.nature

                    if longer.startswith(shorter):
                        right += 1
                        rightd[x] = [y.marked, ins.marked, y, ins]
                    else:
                        wrong[x] = [y.marked, ins.marked, y, ins]

        return wrong, right / tot

    def temp_conu(self, pro, col):
        if hasattr(pro, 'marked'):
            if 'u' in pro.marked:
                idx = pro.marked.index('u')
                if len(col.marked) >= idx:
                    col.marked = replace_at_i(idx, col.marked, '_')
                    col.nature,_ = self.get_code(col.marked, 'c')
        return







if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'all', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    if args[1] == 'lbp':
        ins = long_by_pos()
        ins.begin3()
    elif args[1] == 'cv':
        ins = check_vowels()
        ins.begin()
    elif args[1] == 'cve':
        ins = check_vowels()
        ins.begin_e()
    elif args[1] == 'pc':
        ins = pedecerto()
        ins.begin()

    elif args[1] == 'acc':
        ins = check_vowels()
        ins.begin_ac()

    elif args[1] == 'all':
        ins = long_by_pos()
        ins.begin3()
        ins = check_vowels()
        ins.begin_e()




from bglobals import *


class get_co_lemmas1:
    def __init__(self):
        pass

    def begin(self):
        self.get_atts()
        self.parse_lemmes()
        self.lems_wo_hats()
        self.lemma_wo_hats2()
        self.lem_no_uni()
        self.no_jv()
        self.lem_no_caps()
        self.weed_out()
        self.use_asimil()
        # self.test()
        self.output()
        return

    def get_atts(self):
        self.co_lemmas1 = pi.open_pickle(f'{fold}co_lemmas1', 1)
        lst = to.from_txt2lst(f'{fold}assimilations')
        self.asimil = [x for x in lst if x[0] != '!']
        self.lat_freq = pi.open_pickle(f'{fold}latin_freq', 1)

    def output(self):
        pi.save_pickle(self.co_lemmas1a, f'{fold}co_lemmas1a', 1)


    def test(self):
        co2 = pi.open_pickle(f'{fold}co_lemmas1a', 1)
        if self.co_lemmas1a != co2:
            bb=8
        st = set()
        for k,v in self.co_lemmas1a.items():
            obj = co2[k]
            for x,y in v.items():
                if y != obj[x]:
                    st.add(x)
                    p (y, obj[x])
        return

    def parse_lemmes(self):
        lst = to.from_txt2lst(f'{fold}lemmes')
        lst1 = to.from_txt2lst(f'{fold}lem_ext')
        lst += lst1
        lst = [x for x in lst if x and x[0] != '!']
        lemmas = {}
        ana = []
        repeats = {}
        p (f'parsing the collatinus lemmes sheet')

        for e, x in en(lst):
            vgf.print_intervals(e,1000,None,len(lst))
            x = x.split('|')
            lem = new_lem(x)
            if x[0] in lemmas:
                repeats[x[0]] = lem
            else:
                lemmas[x[0]] = lem

            if lem['lemma'][0].isupper():
                lem['capital'] = 1
            lem['lemma'] = lem['lemma'].lower()
            lem['lemma'] = lem['lemma'].replace(cobr, '')

            if '=' in lem['lemma']:
                lst2 = lem['lemma'].split('=')
                lem["lemma"] = lst2[0]
                lem['syl'] = lst2[1]
            else:
                lem['syl'] = lem['lemma']
                ana.append(lem['lemma'])

        self.co_lemmas0 = lemmas
        return

    def lems_wo_hats(self):
        '''
        the following repeat lemmas can be deleted
        bracteola
        incohibilis
        Malthīnus
        paradoxon
        perlibet
        prendo

        delete first
        propylaeon
        pyrē̆thron
        sparton
        :return:
        '''

        dct = {}
        dct1 = {}
        self.repeats2 = {}
        self.repeats22 = defaultdict(list)
        self.lemmas_wo_hats = {}
        diph3 = ['ae', 'au', 'ei', 'eu', 'oe', 'ui']

        for k, y in self.co_lemmas0.items():
            x = y['lemma']
            if chr(7909) in y['syl']:
                bb = 8
            w = unidecode(x)
            z = remove_hats_diph(x, y['syl'])
            if z != x:
                if any(y in w for y in diph3):
                    dct[x] = z
                else:
                    dct1[x] = z
            self.repeats22[z].append(y)
            if z not in self.lemmas_wo_hats:
                y['lemma'] = z
                self.lemmas_wo_hats[z] = y
            else:
                assert z not in self.repeats2
                self.repeats2[z] = y
        self.repeats22 = {x: y for x, y in self.repeats22.items() if len(y) > 1}
        return

    def lemma_wo_hats2(self):
        self.lemmas_no_hats2 = {}
        self.repeats3 = defaultdict(list)
        for x, y in self.lemmas_wo_hats.items():
            y['spell'] = x
            z = remove_hats(x)
            y['lemma'] = z
            self.repeats3[z].append(y)
            if z not in self.lemmas_no_hats2:
                self.lemmas_no_hats2[z] = y
        self.repeats3 = {x: y for x, y in self.repeats3.items() if len(y) > 1}
        return

    def lem_no_uni(self):
        self.lem_nuni = {}
        self.repeats4 = defaultdict(list)
        for x, y in self.lemmas_wo_hats.items():
            z = unidecode(x)
            y['lemma'] = z
            self.repeats4[z].append(y)
            if z not in self.lem_nuni:
                self.lem_nuni[z] = y
        self.repeats4 = {x: y for x, y in self.repeats4.items() if len(y) > 1}

    def no_jv(self):
        self.lem_jv = {}
        self.i2j = {}
        self.repeats5 = defaultdict(list)
        for x, y in self.lem_nuni.items():
            self.key = x
            self.key = jv.replace(self.key.lower())
            self.key = re.sub(r'[0-9]', '', self.key)
            if x == 'sero':
                bb = 8

            if reg(r'[vj]', x):
                self.i2j_index(y['lemma'].lower())
            self.jv_index(y)
            y['jv'] = y['lemma']
            z = jv.replace(x)
            y['lemma'] = z
            self.repeats5[z].append(y)
            if z not in self.lem_jv:
                self.lem_jv[z] = y
        self.repeats5 = {x: y for x, y in self.repeats5.items() if len(y) > 1}
        pi.save_pickle(self.i2j, f'{fold}i2j', 1)

        return

    def jv_index(self, lem):
        b = 0
        g = lem['geninf'].lower()
        if g:
            lst = lem['geninf'].split(',')
            for x in lst:
                if reg(r'[vj' + conu + r']', x):
                    b += 1
                    x = x.lower()
                    x = unidecode(x)
                    self.i2j_index(x)

        g = lem['perf'].lower()
        if g:
            lst = lem['perf'].split(',')
            for x in lst:
                if reg(r'[vj' + conu + ']', x):
                    x = x.lower()
                    x = unidecode(x)
                    self.i2j_index(x)

    def i2j_index(self, k):
        '''
        only two words belong to a homononymous lemma and
        are sometimes spelled with u or v or i and j
        ('uediiouis', 'uediiou')
        ('ueiouis', 'ueiou')
        neither are common, so they are ignored
        '''

        j = []
        u = []
        w = []
        for e, x in en(k):
            if x == 'j':
                j.append(e)
            elif x == 'v':
                u.append(e)
            elif x == conu:
                w.append(e)
        k = jv.replace(k)
        k = re.sub(r'[0-9]', '', k)
        obj = self.i2j.get((self.key, k))
        if obj and obj != [j, u, w]:
            bb = 8
        else:
            self.i2j[(self.key, k)] = [j, u, w]

    def lem_no_caps(self):
        self.lem_lower = {}
        self.repeats6 = defaultdict(list)
        for x, y in self.lem_jv.items():
            z = x.lower()
            y['lemma'] = z
            self.repeats6[z].append(y)
            if z not in self.lem_lower:
                self.lem_lower[z] = y
        self.repeats6 = {x: y for x, y in self.repeats6.items() if len(y) > 1}
        return

    def weed_out(self):
        cdct = set(self.co_lemmas1.keys())
        dct = {}
        for k, v in self.lem_lower.items():
            if k in cdct:
                dct[k] = v
        self.co_lemmas1a = dct
        return


    def use_asimil(self):
        dct = {}
        dct1 = {}
        for x in self.asimil:
            y = x
            y = unidecode(y)
            lst = x.split(':')
            dct[lst[0]] = lst[1]
            lst = y.split(':')
            dct1[lst[0]] = lst[1]

        self.more_words = {}
        for x, y in self.lem_lower.items():
            for k, v in dct1.items():
                if x.startswith(k):
                    new_word = v + x[len(k):]
                    self.more_words[new_word] = [k]
                    break
        pi.save_pickle(self.more_words, f'{fold}auto_variants')

        return


if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'aw', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    ins = get_co_lemmas1()
    ins.begin()



import pickle, os
from abbreviations import *



def save_pickle(obj, name, any_name=True):
    if not any_name:
        name = mdir + 'pickles/' + name
    elif any_name == 'hi':
        name = mdir + 'hi_pickles/' + name
    elif any_name == 'hit':
        name = mdir + 'pickles/hieroglyphs/tests/' + name


    if not name.endswith(".pkl"):
        name += ".pkl"
    temp = open(name, "wb")
    pickle.dump(obj, temp)
    temp.close()




def open_pickle(name, any_folder=True, warn=True):
    if "DS_Store" in name: return []

    if not name.endswith(".pkl"):
        name += ".pkl"
    if not any_folder:
        name = mdir + 'pickles/' + name
    elif any_folder == 'hi':
        name = mdir + 'hieroglyphs/hi_pickles/' + name
        # name = f'{os.getcwd()}/{name}'

    files_used.add(name)
    pkl_file = open(name, 'rb')
    obj = pickle.load(pkl_file)
    pkl_file.close()

    return obj








import sys,os
import urllib.request
import requests
from setuptools import setup
from pathlib import Path
from setuptools.command.develop import develop
from setuptools.command.install import install

vol2 = os.path.join(os.path.dirname(__file__)) + '/latin/'
vol3 = os.path.join(os.path.dirname(__file__)) + '/latin/general_la/'
sys.path.append(vol2)
sys.path.append(vol3)
fold = f'{vol3}files/'

this_directory = Path(__file__).parent
long_description = ( this_directory/ "README.md").read_text()


def temp():
    #print('running post installation')
    # PUT YOUR POST-INSTALL SCRIPT HERE or CALL A FUNCTION
    s = 'https://storage.googleapis.com/download/storage/v1/b/deduction4/o/on%20cicero.txt?generation=1656578017964238&alt=media'
    urllib.request.urlopen(s)
    if not os.path.exists(fold):
        os.mkdir(fold)
    file = f'{fold}hey.txt'
    r = requests.get(s, stream=True, verify=False)
    if r.status_code == 200:
        r.raw.decode_content = 1
        with open(file, 'wb') as f:
            f.write(r.content)
    else:
        print('failed to download data files')


class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        develop.run(self)
        temp()


class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        temp()

setup(
    name='latin_databases',
    version='0.1.14',
    description='none',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/kylefoley76/latin_databases3',
    author='Kyle Foley',
    author_email='kylefoley202@gmail.com',
    license='BSD',
    zip_safe=False,
    packages=['latin','latin/general_la','latin/unidecode2'],
    install_requires=['Levenshtein',
                      'striprtf==0.0.14',
                      ],
    cmdclass={
            'develop': PostDevelopCommand,
            'install': PostInstallCommand,
        },

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Education',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: MacOS',
        'Programming Language :: Python :: 3.8',
    ],
)

